
;:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
; Blitz2D/3D's Print, Write, Locate and Input replacement for BlitzPlus.
; Created by Teasy <TheChange@yahoo.com> for Hotshot on 08 Jun 2007.
; Last modified on 24 Aug 2010.
;:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
; If you use this in Blitz2D/3D, Input() will not have key-repeat support.
;:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
;
; makes every call to Write and Print show up on screen instantly
; - this may not be suitable in special situations
Global BPL_EternalFlip = True
;
; defaults (should be synced with blitz's default)
Const BPL_Default_x  = 0
Const BPL_Default_y  = 0
Const BPL_Default_br = 0
Const BPL_Default_bg = 0
Const BPL_Default_bb = 0
Const BPL_Default_r  = 255
Const BPL_Default_g  = 255
Const BPL_Default_b  = 255
;
Type BPL_PrintTYPE
    Field x,y
    Field br,bg,bb
    Field r,g,b
End Type
;
Global BPL_Print_.BPL_PrintTYPE
BPL_Init
;
Function BPL_Init()
    BPL_Print_ = New BPL_PrintTYPE
    Locate BPL_Default_x,BPL_Default_y
    pClsColor BPL_Default_br,BPL_Default_bg,BPL_Default_bb , False
    pColor BPL_Default_r,BPL_Default_g,BPL_Default_b , False
End Function
;
Function pClsColor( r=0,g=0,b=0 , native=True )
    BPL_Print_\br = r
    BPL_Print_\bg = g
    BPL_Print_\bb = b
    If native Then ClsColor r,g,b
End Function
;
Function pColor( r=0,g=0,b=0 , native=True )
    BPL_Print_\r = r
    BPL_Print_\g = g
    BPL_Print_\b = b
    If native Then Color r,g,b
End Function
;
Function Locate( x,y )
    BPL_Print_\x = x
    BPL_Print_\y = y
End Function
;
Function Print( msg$="" , cr=True , limity=True,limitx=True )
    ; check if x out of bounds
    If limitx And BPL_Print_\x >= GraphicsWidth()
        BPL_Print_\x = 0
        BPL_Print_\y = BPL_Print_\y + FontHeight()
    End If
    ; new line doesn't fit on screen
    If limity And BPL_Print_\y >= GraphicsHeight() - FontHeight()
        ;BPL_Print_\y = GraphicsHeight() - FontHeight() - 1
        BPL_ScrollScreen
    End If
    If msg <> ""
        Text BPL_Print_\x,BPL_Print_\y , msg
    End If
    ; carriage return (Print)
    If cr
        BPL_Print_\x = 0
        BPL_Print_\y = BPL_Print_\y + FontHeight()
    ; cursor behind text (Write)
    Else
        BPL_Print_\x = BPL_Print_\x + StringWidth( msg )
    End If
    ; show on screen right away
    If BPL_EternalFlip
        Flip False
    End If
End Function
;
Function Write( msg$ )
    Print msg , False
End Function
;
; note: uses pClsColor's set background color
Function BPL_ScrollScreen()
    ; move all so far up that 1 line will fit
    CopyRect 0,BPL_Print_\y-GraphicsHeight()+FontHeight() , GraphicsWidth(),GraphicsHeight() , 0,0
    ; place cursor at last line
    BPL_Print_\y = GraphicsHeight() - FontHeight() - 1
    ; remember current color (minimal system independency)
    Local col_r = ColorRed()
    Local col_g = ColorGreen()
    Local col_b = ColorBlue()
    ; erase last line (with system-set clscolor)
    Color BPL_Print_\br,BPL_Print_\bg,BPL_Print_\bb
    Rect 0,BPL_Print_\y , GraphicsWidth(),FontHeight()
    ; restore color
    Color col_r,col_g,col_b
End Function
;
Function Input$( prompt$="" , value$="" , limit=0 )
    ; place prompt (and move cursor)
    Write prompt
    ; remember current drawing buffer
    Local buffer = GraphicsBuffer()
    ; record original background
    Local back = CreateImage( GraphicsWidth(),FontHeight() )
    SetBuffer ImageBuffer( back )
    CopyRect BPL_Print_\x,BPL_Print_\y , GraphicsWidth(),FontHeight() , 0,0 , buffer,ImageBuffer( back )
    ; go back to original drawing buffer
    SetBuffer buffer
    ; setup input variables
    Local done = False 
    Local key
    Local in$ = value
    Local update
    Local time
    Local lastblink
    Local showcursor
    Local exitloop = False
    ; forget pressed keys
    FlushKeys 
    ; input loop
    Repeat 
        time = MilliSecs()
        If time - lastblink >= 200
            lastblink = time
            showcursor = Not showcursor
            update = True 
        EndIf 
        key = GetKey()
        If key 
            Select key
                Case 27
                    in = ""
                    update = True 
                Case 13  ; enter
                    update = True 
                    exitloop = True 
                Case 8  ; backspace
                    If Len( in ) > 0
                        in = Left( in , Len( in )-1 )
                    EndIf 
                    update = True 
                Default  ; for the rest..
                    ; normal ascii chars only
                    If key >= 32 And key <= 255
                        If limit > 0 And Len( in ) < limit Or limit = 0
                            in = in + Chr( key )
                            update = True
                        End If
                    EndIf 
            End Select 
        EndIf 
        If update
            update = False 
            ; original background
            DrawBlock back , BPL_Print_\x,BPL_Print_\y
            ; overlay entered text
            Text BPL_Print_\x,BPL_Print_\y , in
            ; draw cursor only:
            ; - when visible, and
            ; - when not exiting loop
            If ( showcursor ) And ( Not exitloop )
                Rect BPL_Print_\x+StringWidth( in ),BPL_Print_\y , StringWidth( "W" ),FontHeight()
            End If
            Flip False
        End If
    Until exitloop
    ; forget pressed keys
    FlushKeys
    ; next line
    BPL_Print_\x = 0
    BPL_Print_\y = BPL_Print_\y + FontHeight()
End Function
;
;-------------------------------------------------------------------------------

; demonstration
Function BPL_Demo()
    Local pos,size
    Local msg$
    Local mode = 0 
    Graphics 400,300 , 0,2
    SetBuffer FrontBuffer()
    SeedRnd MilliSecs()
    Locate 0,300-64
    pClsColor 32,48,64
    Cls
    Print "Press <Space> to toggle:"
    Print "- random garbage"
    Print "- manual input"
    Print
    Color 192,224,255
    ;Flip False
    ;Delay 1000
    ;Color 255,0,0
    Repeat 
        ; mode 0: generate garbage
        If mode = 0
            ; new message
            msg = ""
            ; random length
            size = Rand( 1,20 )
            ; fill will garbage
            For pos = 1 To size
                msg = msg + Chr( Rand( 32,128 ) )
            Next 
            ; random print or write
            If Rand( 0,1 )
                Print msg
            Else 
                Write msg
            EndIf 
            ; hold for a bit
            Delay 500
        ; mode 1: manual input
        ElseIf mode = 1
            ; request message
            msg = Input( ">" )
            ; auto change back to garbage mode
            ; so you can change mode again
            mode = 0
        EndIf 
        ; space: change mode
        If KeyHit( 57 )
            mode = Not mode
        EndIf 
        Flip False
    Until KeyHit( 1 )
    End 
End Function

;BPL_Demo

;~IDEal Editor Parameters:
;~C#BlitzPlus145