
;:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
; Print, Write, Locate, Input replacement for Blitz 3D
; - Use version "1b" for BlitzPlus (supports key-repeat in Input/GetKey natively)
; Created by Teasy <TheChange@yahoo.com>
; Last modified on 02 Nov 2010
;:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
; Uses "Input.Def.BB"
; Uses API: Windows Clipboard (see further below)
;:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
; * 01 dec 2010
;   / added multi-core delay in pInput()
;   / made correction to ScrollScreen, to be pixel-perfect (i hope)
; * 02 nov 2010
;   / copied file to library folder;
;     this is now the main edit version of blitz print lib for blitz2d/3d
; * 22 oct 2010
;   / add support for clipboard in input function
;     - paste with shift+insert (scancode) or ctrl+v (ascii)
;   / added keyrepeat support for backspace, other keys single
;   ~ add keyrepeat support in input function (Blitz2D/3D)
;     - separate toggle, supports only the "qwerty" standard
;   / remove forced override; allows naming uniformity
;   / add a multitude of configurable options
;:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
;
; make every call to Write/Print and/or Input show up on screen instantly
; - suitable for switching between front/backbuffer
; - setting a value of -1 (auto-detect) will flip when drawing to the backbuffer
; - default -1 (for both)
Global BPL_PrintFlips = -1
Global BPL_InputFlips = -1
;
; use vertical sync on any flip (default off)
Global BPL_FlipVSync = False
;
; add a delay after a flip (default on)
; - recommended delay time: 1..10 ms (default 1 ms)
Global BPL_FlipDelay = True
Global BPL_FlipDelayTime = 1
;
; support for directx's directinput feature (default off)
; - use a value of -1 for blitz' default
Global BPL_DirectInput = False
;
; enable support for pasting text in 'pInput' function (default on)
Global BPL_ClipboardSupport = True
;
; custom key repeat feature ("qwerty" lay-out only)
; - default off for maximum compatibility
; - default delay 250ms and interval 40ms
; ! implemented for 'backspace' only
Global BPL_KeyRepeat = True
Global BPL_KeyRepeatDelay = 250
Global BPL_KeyRepeatInterval = 40
;
; defaults (should be synced with blitz's default)
Const BPL_Default_x  = 0
Const BPL_Default_y  = 0
Const BPL_Default_br = 0
Const BPL_Default_bg = 0
Const BPL_Default_bb = 0
Const BPL_Default_r  = 255
Const BPL_Default_g  = 255
Const BPL_Default_b  = 255
;
Type BPL_PrintTYPE
    Field x,y
    Field br,bg,bb
    Field r,g,b
    Field key,keystart,keyrepeat
End Type
;
Global BPL_Print_.BPL_PrintTYPE
BPL_Init
;
Function BPL_Init()
    BPL_Print_ = New BPL_PrintTYPE
    pLocate BPL_Default_x,BPL_Default_y
    pClsColor BPL_Default_br,BPL_Default_bg,BPL_Default_bb , False
    pColor BPL_Default_r,BPL_Default_g,BPL_Default_b , False
    EnableDirectInput BPL_DirectInput
End Function
;
Function pClsColor( r=0,g=0,b=0 , native=True )
    BPL_Print_\br = r
    BPL_Print_\bg = g
    BPL_Print_\bb = b
    If native Then ClsColor r,g,b
End Function
;
Function pCls( r=-1,g=-1,b=-1 , native=True )
    If native
        If b<>-1
            pClsColor r,g,b
        End If
        Cls
    Else
        If b=-1
            Color BPL_Print_\br,BPL_Print_\bg,BPL_Print_\bb
        Else
            Color r,g,b
        End If
        Rect 0,0 , GraphicsWidth(),GraphicsHeight()
        Color BPL_Print_\r,BPL_Print_\g,BPL_Print_\b
    End If
End Function
;
Function pColor( r=0,g=0,b=0 , native=True )
    BPL_Print_\r = r
    BPL_Print_\g = g
    BPL_Print_\b = b
    If native Then Color r,g,b
End Function
;
Function pLocate( x,y )
    BPL_Print_\x = x
    BPL_Print_\y = y
End Function
;
Function pPrint( msg$="" , cr=True , limity=True,limitx=True )
    Local gw = GraphicsWidth()
    Local gh = GraphicsHeight()
    Local fh = FontHeight()
    ; check if x out of bounds
    If limitx And BPL_Print_\x >= gw
        BPL_Print_\x = 0
        BPL_Print_\y = BPL_Print_\y + fh
    End If
    ; new line doesn't fit on screen
    If limity And BPL_Print_\y >= gh - fh
        ;BPL_Print_\y = GraphicsHeight() - FontHeight() - 1
        BPL_ScrollScreen
    End If
    If msg <> ""
        Text BPL_Print_\x,BPL_Print_\y , msg
    End If
    ; carriage return (Print)
    If cr
        BPL_Print_\x = 0
        BPL_Print_\y = BPL_Print_\y + fh
    ; cursor behind text (Write)
    Else
        BPL_Print_\x = BPL_Print_\x + StringWidth( msg )
    End If
    ; show on screen right away
    If BPL_PrintFlips = -1
        If GraphicsBuffer() = BackBuffer()
            Flip BPL_FlipVSync
            If BPL_FlipDelay
                Delay BPL_FlipDelayTime
            End If
        End If
    ElseIf BPL_PrintFlips = True
        Flip BPL_FlipVSync
        If BPL_FlipDelay
            Delay BPL_FlipDelayTime
        End If
    End If
End Function
;
Function pWrite( msg$ )
    pPrint msg , False
End Function
;
; note: uses pClsColor's set background color
Function BPL_ScrollScreen()
    Local gw = GraphicsWidth()
    Local gh = GraphicsHeight()
    Local fh = FontHeight()
    ; move all so far up that 1 line will fit
    CopyRect 0,BPL_Print_\y-gh+fh , gw,gh , 0,0
    ; place cursor at last line
    BPL_Print_\y = gh-fh;-1
    ; remember current color (minimal system independency)
    ;Local col_r = ColorRed()
    ;Local col_g = ColorGreen()
    ;Local col_b = ColorBlue()
    ; erase last line (with system-set clscolor)
    Color BPL_Print_\br,BPL_Print_\bg,BPL_Print_\bb
    Rect 0,BPL_Print_\y , gw,fh
    ; restore color
    ;Color col_r,col_g,col_b
    Color BPL_Print_\r,BPL_Print_\g,BPL_Print_\b
End Function
;
Function pInput$( prompt$="" , value$="" , limit=0 )
    Local gw = GraphicsWidth()
    Local fh = FontHeight()
    Local sww = StringWidth( "W" )
    ; place prompt (and move cursor)
    pWrite prompt
    ; remember current drawing buffer
    Local buffer = GraphicsBuffer()
    ; record original background
    Local back = CreateImage( gw,fh )
    SetBuffer ImageBuffer( back )
    CopyRect BPL_Print_\x,BPL_Print_\y , gw,fh , 0,0 , buffer,ImageBuffer( back )
    ; go back to original drawing buffer
    SetBuffer buffer
    ; setup input variables
    Local done = False 
    Local key
    Local in$ = value
    Local update
    Local time
    Local lastblink
    Local showcursor
    Local exitloop = False
    Local pastetext
    ; forget pressed keys
    FlushKeys 
    ; input loop
    Repeat 
        time = MilliSecs()
        If time - lastblink >= 200
            lastblink = time
            showcursor = Not showcursor
            update = True 
        EndIf
        Repeat
            key = GetKey()
            If key
                Select key
                    Case 22  ; ctrl+V
                        pastetext = True
                    Case 27  ; esc
                        in = ""
                        update = True
                    Case 13  ; enter
                        update = True
                        exitloop = True
                    Default  ; for the rest..
                        ; normal ascii chars only
                        If key >= 32 And key <= 255
                            If limit > 0 And Len( in ) < limit Or limit = 0
                                in = in + Chr( key )
                                update = True
                            End If
                        EndIf
                End Select
            EndIf
        Until key = 0
        
        ; backspace (with key-repeat)
        Local exec_backspace
        If KeyHit( key_BackSpace )
            BPL_Print_\keystart = time
            exec_backspace = True
        ElseIf KeyDown( key_BackSpace )
            If BPL_Print_\keyrepeat
                If time - BPL_Print_\keystart >= BPL_KeyRepeatInterval
                    BPL_Print_\keystart = time
                    exec_backspace = True
                End If
            Else
                If time - BPL_Print_\keystart >= BPL_KeyRepeatDelay
                    BPL_Print_\keystart = time
                    BPL_Print_\keyrepeat = True
                    exec_backspace = True
                End If
            End If
        Else
            If BPL_Print_\keyrepeat
                BPL_Print_\keyrepeat = False
            End If
        End If
        If exec_backspace
            exec_backspace = False
            If Len( in ) > 0
                in = Left( in , Len( in )-1 )
            EndIf
            update = True
        End If
        
        ; paste from clipboard (Shift + Insert, Ctrl + V)
        If pastetext = False
            If ( ( KeyDown( key_LeftShift ) Or KeyDown( key_RightShift ) ) And KeyHit( key_Insert ) ) ;Or ( ( KeyDown( key_LeftCtrl ) Or KeyDown( key_RightCtrl ) ) And KeyHit( key_V ) )
                pastetext = True
            End If
        End If
        If pastetext
            pastetext = False
            Local bigtext$ = GetClipboard()
            ; paste the text in the input field until:
            ; - find the first control character (if any) (in text-to-paste)
            ; - a limit was set (parameter) and reached
            Local length = Len( bigtext )
            Local pos
            For pos = 1 To length
                If limit > 0 And Len( in ) = limit
                    Exit
                End If
                Local char$ = Mid( bigtext , pos,1 )
                If Asc( char ) < 32
                    Exit
                End If
                in = in + char
            Next
        End If
        
        ; visual refresh
        If update
            update = False
            ; original background
            DrawBlock back , BPL_Print_\x,BPL_Print_\y
            ; overlay entered text
            Text BPL_Print_\x,BPL_Print_\y , in
            ; draw cursor only:
            ; - when visible, and
            ; - when not exiting loop
            If ( showcursor ) And ( Not exitloop )
                Rect BPL_Print_\x+StringWidth( in ),BPL_Print_\y , sww,fh
            End If
            If BPL_InputFlips = -1
                If buffer = BackBuffer()
                    Flip BPL_FlipVSync
                    If BPL_FlipDelay
                        Delay BPL_FlipDelayTime
                    End If
                End If
            ElseIf BPL_InputFlips = True
                Flip BPL_FlipVSync
                If BPL_FlipDelay
                    Delay BPL_FlipDelayTime
                End If
            End If
        End If
        
        ; multi-core time
        Delay 1
    Until exitloop
    ; forget pressed keys
    FlushKeys
    ; next line
    BPL_Print_\x = 0
    BPL_Print_\y = BPL_Print_\y + fh
    ; return current input value
    Return in
End Function
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
;   Windows clipboard
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; file: userlibs\user32.decl
;-------------------------------------------------------------------------------
;   .lib "USER32.DLL"
;
;   api_OpenClipboard%( hwnd% ) : "OpenClipboard"
;   api_CloseClipboard%() : "CloseClipboard"
;   api_ExamineClipboard%( format% ) : "IsClipboardFormatAvailable"
;   api_EmptyClipboard%() : "EmptyClipboard"
;   api_GetClipboardData$( format% ) : "GetClipboardData"
;   api_SetClipboardData%( format% , txt$ ) : "SetClipboardData"
;   api_SetClipboardData_Direct%( format% , Handle* ) : "SetClipboardData"
;-------------------------------------------------------------------------------
;
Function SetClipboard( Text$ )
	If Text = "" Then Return
	If api_OpenClipboard( 0 )
		api_EmptyClipboard
		api_SetClipboardData 1 , Text
		api_CloseClipboard
	EndIf
End Function
;
Function GetClipboard$()
	Local Text$
    
	If api_OpenClipboard( 0 )
		If api_ExamineClipboard( 1 )
			Text = api_GetClipboardData( 1 )
		EndIf
		api_CloseClipboard
	EndIf
    
	Return Text
End Function
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
;   demonstration
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
Function BPL_Demo()
    Local pos,size
    Local msg$
    Local mode = 0 
    Graphics 400,300 , 0,2
    SetBuffer BackBuffer()
    SeedRnd MilliSecs()
    pLocate 0,300-64
    pClsColor 0,64,128
    pCls
    pColor 255,255,255
    pPrint "Press <Space> to toggle:"
    pPrint "- random garbage"
    pPrint "- manual input"
    pPrint
    pColor 192,224,255
    Repeat 
        ; mode 0: generate garbage
        If mode = 0
            ; new message
            msg = ""
            ; random length
            size = Rand( 1,20 )
            ; fill will garbage
            For pos = 1 To size
                msg = msg + Chr( Rand( 32,128 ) )
            Next 
            ; random print or write
            If Rand( 0,1 )
                pPrint msg
            Else 
                pWrite msg
            EndIf 
            ; hold for a bit
            Delay 500
        ; mode 1: manual input
        ElseIf mode = 1
            ; request message
            pColor 255,255,255
            msg = pInput( ">" , "" , 30 )
            pColor 192,224,255
            ; auto change back to garbage mode
            ; so you can change mode again
            mode = 0
        EndIf 
        ; space: change mode
        If KeyHit( 57 )
            mode = Not mode
        EndIf 
        Flip False
    Until KeyHit( 1 )
    End 
End Function
;
;BPL_Demo

;~IDEal Editor Parameters:
;~C#Blitz3D1105