
;-------------------------------------------------------------------------------
; Data Included library - Created by Teasy <TheChange@yahoo.com>

; Usage:
; * Convert file to data:
;   di_Compact Source_File$ , Target_File$ , Label$ ,
;                Silent = False , Abortable = True
; * Convert data to file:
;   di_Extract Target_File$ ,
;                Silent = True , Abortable = False


Const di_UpdateCycles =  1000
Const di_UpdateDelay  =  1000


; Convert a file to a .BB data include with specific label
Function di_Compact( Source_File$,Target_File$ , Label$ , Silent=False,Abortable=True )
    Local Size
    Local Source
    Local Target
    Local Buffer
    Local DataLine$
    Local Bytes
    Local Byte
    Local Cycles
    Local LastUpdate
    Local MilliSecs

    If Not Silent
        Print "Source (Binary): "+Source_File
        Print "Target (Blitz):  "+Target_File
    End If

    ; make sure source exists
    If FileType( Source_File ) <> 1
        RuntimeError "File not found: "+Chr(13)+Source_File
    EndIf

    ; amount of data to copy
    Size = FileSize( Source_File )
    Buffer = CreateBank( Size+1 )

    ; attempt to read from source
    Source = ReadFile( Source_File )
    If Not Source
        RuntimeError "Unable to open file: "+Chr(13)+Source_File
    EndIf

    ; attempt to write to target
    Select FileType( Target_File )
    Case 0
        ; create target if non-existing
        Target = WriteFile( Target_File )
    Case 1
        ; open target (for appending) if existing
        Target = OpenFile( Target_File )
        ; and move to end of file
        SeekFile Target , FileSize( Target_File )
    End Select

    ; verify target
    If Target = 0
        RuntimeError "Unable to create file: "+Chr(13)+Target_File
    EndIf

    ; write header
    WriteLine Target , ""
    WriteLine Target , ""
    WriteLine Target , "."+Label
    WriteLine Target , "Data "+Size

    If Not Silent
        Print "Reading "+Size+" Bytes"
    End If

    ; copy entire file to memory
    ReadBytes Buffer,Source , 0,Size
    CloseFile Source

    If Not Silent
        Print "Converting"
    End If

    DataLine = "Data "
    Bytes = 0

    For Byte = 0 To Size-1
        ; read from memory
        DataLine = DataLine + PeekByte( Buffer , Byte )
        Bytes = Bytes + 1
        ; after each 32 bytes
        If Bytes = 32
            ; write line to target
            WriteLine Target , DataLine
            DataLine = "Data "
            Bytes = 0

            If Not Silent
                Cycles = Cycles + 1
                If Cycles = di_UpdateCycles
                    Cycles = 0
                    MilliSecs = MilliSecs()
                    If MilliSecs - LastUpdate >= di_UpdateDelay
                        LastUpdate = MilliSecs
                        Print ( Byte * 100 / Size ) + "%"
                    End If
                End If
                If Abortable
                    If KeyHit( 1 )
                        Exit
                    End If
                EndIf
            End If
        Else
            DataLine = DataLine + ","
        End If
    Next

    ; write last remaining line if any data on it
    If DataLine <> "Data "
        WriteLine Target , Left ( DataLine , Len ( DataLine ) - 1 )  ; Remove trailing comma
    End If

    CloseFile Target
    FreeBank Buffer

    Return True
End Function


; Extract data from current data reading position to Target_File
Function di_Extract( Target_File$ , Silent=True , Abortable=False )
    Local Target
    Local Size
    Local Buffer
    Local Byte
    Local Value
    Local Cycles
    Local LastUpdate
    Local MilliSecs

    If Not Silent
        Print "Source: " + Size + " Bytes"
        Print "Target: " + Target_File
    End If

    Read Size
    Buffer = CreateBank( Size+1 )

    Target = WriteFile( Target_File )
    If Not Target
        RuntimeError "Unable to create file: "+Chr(13)+Target_File
    EndIf

    If Not Silent
        Print "Reading"
    End If

    ; data -> bank
    For Byte = 0 To Size-1
        Read Value
        PokeByte Buffer , Byte,Value

        If Not Silent
            Cycles = Cycles + 1
            If Cycles = di_UpdateCycles
                Cycles = 0
                MilliSecs = MilliSecs()
                If MilliSecs - LastUpdate >= di_UpdateDelay
                    LastUpdate = MilliSecs
                    Print ( Byte * 100 / Size ) + "%"
                End If
            End If
            If Abortable
                If KeyHit( 1 )
                    Exit
                End If
            EndIf
        End If
    Next

    If Not Silent
        Print "Saving"
    End If

    ; bank -> file
    WriteBytes Buffer,Target , 0,Size

    CloseFile Target
    FreeBank Buffer

    Return True
End Function

