
; graphics resolution selector v2 (2007)
; created by teasy <thechange@yahoo.com>
; usage:
;
;   omit parameters to keep current resolution to start selecting resolution with
;
;       SelectGraphicsMode
;
;   or specify width, height, colordepth and optionally windowmode (default fullscreen)
;
;       SelectGraphicsMode 640,480,16
;
;   afterwards a selected resolution is still in place,
;   and you can use the Blitz functions to acquire the data
;
;       RuntimeError GraphicsWidth()+"x"+GraphicsHeight()+"x"+GraphicsDepth()
;
; 100% independent module
; based on gfx selector v1 (2002)



Type SelectGraphicsMode_GfxMode
    Field SelectorFont
    Field TitleFont
    Field HugeFont
End Type



Function SelectGraphicsMode_StartGraphics.SelectGraphicsMode_GfxMode( w,h , d,m )  ; width,height , depth,mode
    Local p.SelectGraphicsMode_GfxMode

    ; setup
    p = New SelectGraphicsMode_GfxMode
    Graphics w,h , d,m
    SetBuffer BackBuffer()
    ClsColor 0,48,0
    p\SelectorFont = LoadFont ( "FixedSys" )
    p\TitleFont = LoadFont ( "Terminal" , 12 )
    p\HugeFont = LoadFont ( "FixedSys" , 64 )

    Return p
End Function



Function SelectGraphicsMode_EndGraphics( p.SelectGraphicsMode_GfxMode )

    ; release
    FreeFont p\HugeFont
    FreeFont p\TitleFont
    FreeFont p\SelectorFont

    ; remove
    Delete p

    ; reset background
    ClsColor 0,0,0

End Function



Function SelectGraphicsMode( w=0,h=0 , d=0,m=1 )
    Local p.SelectGraphicsMode_GfxMode

    Local Counter
    Local Current
    Local Cursor
    Local Count = CountGfxModes()
    Local Text$
    Local Done = False
    Local TextPositionY
    Local Testing = False
    Local TestTime
    Local TestTimeOut = 4000
    Local Old
    Local OldGraphicsWidth
    Local OldGraphicsHeight
    Local OldGraphicsDepth

    Local Enter  = False
    Local Escape = False
    Local Up     = False
    Local Down   = False

    ; setup initial graphics mode
    
    ; - don't change resolution (keep current)
    If w = 0 And h = 0 And d = 0
        p = SelectGraphicsMode_StartGraphics( GraphicsWidth(),GraphicsHeight() , GraphicsDepth(),m )
    ; - start with specific resolution
    Else 
        p = SelectGraphicsMode_StartGraphics( w,h , d,m )
    EndIf 

    ; - find number of current resolution in list
    For Counter = 1 To Count
        If GfxModeWidth(Counter)=GraphicsWidth() And GfxModeHeight(Counter)=GraphicsHeight() And GfxModeDepth(Counter)=GraphicsDepth()
            Cursor = Counter
            Current = Cursor
            Exit
        EndIf
    Next

    ; invoke resolution selector

    Repeat

        ; input

        Up     = KeyHit( 200 )
        Down   = KeyHit( 208 )
        Enter  = KeyHit(  28 )
        Escape = KeyHit(   1 )

        If Testing
            ; escape or time out -> abort (previous)
            If Escape Or MilliSecs() - TestTime > TestTimeOut
                ; blank
                ClsColor 0,0,0
                Cls
                Flip
                ; release
                SelectGraphicsMode_EndGraphics p
                ; setup (back to previous)
                p = SelectGraphicsMode_StartGraphics( OldGraphicsWidth,OldGraphicsHeight , OldGraphicsDepth,m )
                ; done testing
                Testing = False
            EndIf
            ; enter -> confirm
            If Enter
                Current = Cursor
                Testing = False
            EndIf
        ; not testing
        Else
            ; escape -> quit
            Done = Escape
            ; up -> previous resolution
            If Up
                Cursor = Cursor - 1
                If Cursor <= 0
                    Cursor = Count
                EndIf
            EndIf
            ; down -> next resolution
            If Down
                Cursor = Cursor + 1
                If Cursor > Count
                    Cursor = 1
                EndIf
            EndIf
            ; enter -> test resolution
            If (Enter And (Not Cursor = Current))
                ; remember current
                OldGraphicsWidth  = GraphicsWidth()
                OldGraphicsHeight = GraphicsHeight()
                OldGraphicsDepth  = GraphicsDepth()
                ; blank
                ClsColor 0,0,0
                Cls
                Flip
                ; release
                SelectGraphicsMode_EndGraphics p
                ; setup (try new)
                p = SelectGraphicsMode_StartGraphics( GfxModeWidth(Cursor),GfxModeHeight(Cursor) , GfxModeDepth(Cursor),m )
                ; start testing
                Testing = True
                TestTime = MilliSecs ( )
            EndIf
        EndIf  ; testing

        ; titles

        Color 0,192,0
        SetFont p\TitleFont
        If Testing
            Text GraphicsWidth()/2,FontHeight() , "C O N F I R M   G R A P H I C S   M O D E" , True,True
        Else
            Text GraphicsWidth()/2,FontHeight() , "S E L E C T   G R A P H I C S   M O D E" , True,True
        EndIf
        If Testing
            Text GraphicsWidth()/2,GraphicsHeight()-FontHeight() , "ENTER: KEEP   ESCAPE: UNDO" , True,True
        Else
            Text GraphicsWidth()/2,GraphicsHeight()-FontHeight() , "UP/DOWN: SELECT   ENTER: TEST   ESCAPE: EXIT" , True,True
        EndIf

        ; resolutions

        SetFont p\SelectorFont
        Color 0,96,0
        Rect 0,0 , GraphicsWidth(),GraphicsHeight() , False
        Color 0,64,0
        Rect GraphicsWidth()/4,FontHeight()*1.5 , GraphicsWidth()/2,GraphicsHeight()-FontHeight()*3 , False

        If Testing
            SetFont p\HugeFont
            Shade = ( MilliSecs() - TestTime ) * 255 / TestTimeOut
            Color Shade,255-Shade,0
            Text GraphicsWidth()/2,GraphicsHeight()/2 , Int( Floor( ( TestTimeOut - Float(MilliSecs()-TestTime) ) / 1000 + 0.5 ) ) , True,True
        Else
            SetFont p\SelectorFont
            For Counter = 1 To Count
                TextPositionY = GraphicsHeight()/2 - Cursor*FontHeight() + Counter*FontHeight()
                If TextPositionY > FontHeight()*2 And TextPositionY < GraphicsHeight()-FontHeight()*2
                    Text = GfxModeWidth ( Counter ) + "x" + GfxModeHeight ( Counter ) + "x" + GfxModeDepth ( Counter )
                    Color 0,128,0
                    If Cursor  = Counter
                        Color 0,224,0
                    EndIf
                    If Current = Counter
                        Color 128,192,0
                    EndIf
                    Text GraphicsWidth()/2,TextPositionY , Text , True,True
                EndIf
            Next
        EndIf

        Flip
        Delay 10
        Cls

    Until Done

    SelectGraphicsMode_EndGraphics p

End Function

