;-------------------------------------------------------------------------------
;
;   Input Definitions (Mouse, keyboard)
;
;-------------------------------------------------------------------------------
; 07 Nov 2011
; - swapped pause with numlock, as apparently this is the 'new' 'standard'
; - added arrays and functions for finding or converting upper/lower case ASCII keys (1=!)
; 06 Nov 2011
; - added Key_ShortName() to provide a replica of the keyboard glyphs (for small spaces)
; - made some fixes to key values
; - added virtual values for keylock states ('lights')
; 03 Nov 2011
; - made formatting more compatible with ideal
; - added custom key constants for keyboard layouts: none, unknown, invalid
; - added printscreen and other special keys
; 27 Feb 2010
; - Changed visual name 'Pad ..' to 'NumPad ..'
; 02 Jan 2007
; - Last known edit (unknown)
;-------------------------------------------------------------------------------

Const Key_MaxKeys = 1024
Local DefInput_Number

;======================================================================================================================;
;  Numeric names

Dim Numeric_Name$ ( Key_MaxKeys )
: Numeric_Name (  0 ) = "Zero"
: Numeric_Name (  1 ) = "One"
: Numeric_Name (  2 ) = "Two"
: Numeric_Name (  3 ) = "Three"
: Numeric_Name (  4 ) = "Four"
: Numeric_Name (  5 ) = "Five"
: Numeric_Name (  6 ) = "Six"
: Numeric_Name (  7 ) = "Seven"
: Numeric_Name (  8 ) = "Eight"
: Numeric_Name (  9 ) = "Nine"
: Numeric_Name ( 10 ) = "Ten"
: Numeric_Name ( 11 ) = "Eleven"
: Numeric_Name ( 12 ) = "Twelve"

For DefInput_Number = 13 To 255
    Numeric_Name ( DefInput_Number ) = DefInput_Number
Next

;======================================================================================================================;
;  Mouse button values and names

Const Button_Left%   = 1
Const Button_Right%  = 2
Const Button_Middle% = 3

Dim Button_Name$ ( 255 )
: Button_Name ( 0 ) = "Button " + Numeric_Name ( 0 )
: Button_Name ( Button_Left ) = "Left Button"
: Button_Name ( Button_Right ) = "Right Button"
: Button_Name ( Button_Middle ) = "Middle Button"

For DefInput_Number = 4 To 255
    Button_Name ( DefInput_Number ) = "Button " + Numeric_Name ( DefInput_Number )
Next

;======================================================================================================================;
;  Joystick button names

Dim Joy_Name$ ( 255 )
: Joy_Name (  0 ) = "Button " + Numeric_Name ( 0 )
: Joy_Name (  1 ) = "Primary Button"
: Joy_Name (  2 ) = "Secondary Button"
: Joy_Name (  3 ) = "Tertiary Button"
: Joy_Name (  4 ) = "Quaternary Button"
: Joy_Name (  5 ) = "Quinary Button"
: Joy_Name (  6 ) = "Senary Button"
: Joy_Name (  7 ) = "Septenary Button"
: Joy_Name (  8 ) = "Octonary Button"
: Joy_Name (  9 ) = "Nonary Button"
: Joy_Name ( 10 ) = "Denary Button"
: Joy_Name ( 11 ) = "Button Eleven"
: Joy_Name ( 12 ) = "Duodenary Button"

For DefInput_Number = 13 To 255
    Joy_Name ( DefInput_Number ) = "Button " + Numeric_Name ( DefInput_Number )
Next

;======================================================================================================================;
;  Scancodes

;-( Numeric )--------------------------------------------;

Const Key_1% =  2
Const Key_2% =  3
Const Key_3% =  4
Const Key_4% =  5
Const Key_5% =  6
Const Key_6% =  7
Const Key_7% =  8
Const Key_8% =  9
Const Key_9% = 10
Const Key_0% = 11

Dim Key_Num% ( 9 )
: Key_Num ( 0 ) = Key_0
: Key_Num ( 1 ) = Key_1
: Key_Num ( 2 ) = Key_2
: Key_Num ( 3 ) = Key_3
: Key_Num ( 4 ) = Key_4
: Key_Num ( 5 ) = Key_5
: Key_Num ( 6 ) = Key_6
: Key_Num ( 7 ) = Key_7
: Key_Num ( 8 ) = Key_8
: Key_Num ( 9 ) = Key_9

;-( Alphabetic )-----------------------------------------;

Const Key_Q% = 16
Const Key_W% = 17
Const Key_E% = 18
Const Key_R% = 19
Const Key_T% = 20
Const Key_Y% = 21
Const Key_U% = 22
Const Key_I% = 23
Const Key_O% = 24
Const Key_P% = 25
Const Key_A% = 30
Const Key_S% = 31
Const Key_D% = 32
Const Key_F% = 33
Const Key_G% = 34
Const Key_H% = 35
Const Key_J% = 36
Const Key_K% = 37
Const Key_L% = 38
Const Key_Z% = 44
Const Key_X% = 45
Const Key_C% = 46
Const Key_V% = 47
Const Key_B% = 48
Const Key_N% = 49
Const Key_M% = 50

Dim Key_Alpha% ( 26 )
: Key_Alpha (  1 ) = Key_A
: Key_Alpha (  2 ) = Key_B
: Key_Alpha (  3 ) = Key_C
: Key_Alpha (  4 ) = Key_D
: Key_Alpha (  5 ) = Key_E
: Key_Alpha (  6 ) = Key_F
: Key_Alpha (  7 ) = Key_G
: Key_Alpha (  8 ) = Key_H
: Key_Alpha (  9 ) = Key_I
: Key_Alpha ( 10 ) = Key_J
: Key_Alpha ( 11 ) = Key_K
: Key_Alpha ( 12 ) = Key_L
: Key_Alpha ( 13 ) = Key_M
: Key_Alpha ( 14 ) = Key_N
: Key_Alpha ( 15 ) = Key_O
: Key_Alpha ( 16 ) = Key_P
: Key_Alpha ( 17 ) = Key_Q
: Key_Alpha ( 18 ) = Key_R
: Key_Alpha ( 19 ) = Key_S
: Key_Alpha ( 20 ) = Key_T
: Key_Alpha ( 21 ) = Key_U
: Key_Alpha ( 22 ) = Key_V
: Key_Alpha ( 23 ) = Key_W
: Key_Alpha ( 24 ) = Key_X
: Key_Alpha ( 25 ) = Key_Y
: Key_Alpha ( 26 ) = Key_Z

;-( Numerical keypad )-----------------------------------;

Const Key_Pad7% = 71
Const Key_Pad8% = 72
Const Key_Pad9% = 73
Const Key_Pad4% = 75
Const Key_Pad5% = 76
Const Key_Pad6% = 77
Const Key_Pad1% = 79
Const Key_Pad2% = 80
Const Key_Pad3% = 81
Const Key_Pad0% = 82

Dim Key_Pad% ( 9 )
: Key_Pad ( 0 ) = Key_Pad0
: Key_Pad ( 1 ) = Key_Pad1
: Key_Pad ( 2 ) = Key_Pad2
: Key_Pad ( 3 ) = Key_Pad3
: Key_Pad ( 4 ) = Key_Pad4
: Key_Pad ( 5 ) = Key_Pad5
: Key_Pad ( 6 ) = Key_Pad6
: Key_Pad ( 7 ) = Key_Pad7
: Key_Pad ( 8 ) = Key_Pad8
: Key_Pad ( 9 ) = Key_Pad9

;-( Function keys )--------------------------------------;

Const Key_Function1%  = 59
Const Key_Function2%  = 60
Const Key_Function3%  = 61
Const Key_Function4%  = 62
Const Key_Function5%  = 63
Const Key_Function6%  = 64
Const Key_Function7%  = 65
Const Key_Function8%  = 66
Const Key_Function9%  = 67
Const Key_Function10% = 68
Const Key_Function11% = 87
Const Key_Function12% = 88

Dim Key_Function% ( 12 )
: Key_Function (  1 ) = Key_Function1
: Key_Function (  2 ) = Key_Function2
: Key_Function (  3 ) = Key_Function3
: Key_Function (  4 ) = Key_Function4
: Key_Function (  5 ) = Key_Function5
: Key_Function (  6 ) = Key_Function6
: Key_Function (  7 ) = Key_Function7
: Key_Function (  8 ) = Key_Function8
: Key_Function (  9 ) = Key_Function9
: Key_Function ( 10 ) = Key_Function10
: Key_Function ( 11 ) = Key_Function11
: Key_Function ( 12 ) = Key_Function12

Const Key_F1%  = Key_Function1
Const Key_F2%  = Key_Function2
Const Key_F3%  = Key_Function3
Const Key_F4%  = Key_Function4
Const Key_F5%  = Key_Function5
Const Key_F6%  = Key_Function6
Const Key_F7%  = Key_Function7
Const Key_F8%  = Key_Function8
Const Key_F9%  = Key_Function9
Const Key_F10% = Key_Function10
Const Key_F11% = Key_Function11
Const Key_F12% = Key_Function12

;-( Shift states )---------------------------------------;

Const Key_LeftShift%  =  42
Const Key_RightShift% =  54
Const Key_LeftAlt%    =  56
Const Key_RightAlt%   = 184
Const Key_LeftCtrl%   =  29
Const Key_RightCtrl%  = 157

Const Key_CapsLock%   =  58
Const Key_NumLock%    = 197 ;69 (ori/docs)
Const Key_ScrollLock% =  70

; alternative names
Const Key_LeftAlternate%  = Key_LeftAlt
Const Key_RightAlternate% = Key_RightAlt
Const Key_LeftControl%    = Key_LeftCtrl
Const Key_RightControl%   = Key_RightCtrl

Const Key_ShiftLeft%  = Key_LeftShift
Const Key_ShiftRight% = Key_RightShift
Const Key_AltLeft%    = Key_LeftAlt
Const Key_AltRight%   = Key_RightAlt
Const Key_CtrlLeft%   = Key_LeftCtrl
Const Key_CtrlRight%  = Key_RightCtrl

;-( Control keys )---------------------------------------;

Const Key_Escape%       =   1
Const Key_Tab%          =  15
Const Key_BackSpace%    =  14
Const Key_Space%        =  57
Const Key_Enter%        =  28

Const Key_Insert%       = 210
Const Key_Delete%       = 211
Const Key_Home%         = 199
Const Key_End%          = 207
Const Key_PageUp%       = 201
Const Key_PageDown%     = 209

Const Key_PrintScreen%  = 183
Const Key_Pause%        =  69 ;197 (ori/docs)

Const Key_LeftWindows%  = 219
Const Key_RightWindows% = 220
Const Key_Apps%         = 221

; alternative names
Const Key_Esc%          = Key_Escape
Const Key_BkSp%         = Key_BackSpace
Const Key_SpaceBar%     = Key_Space
Const Key_Return%       = Key_Enter

Const Key_Del%          = Key_Delete
Const Key_PgUp%         = Key_PageUp
Const Key_PgDn%         = Key_PageDown

Const Key_PrtScn%       = Key_PrintScreen
Const Key_SysReq%       = Key_PrintScreen
Const Key_Break%        = Key_Pause

Const Key_LeftWin%      = Key_LeftWindows
Const Key_RightWin%     = Key_RightWindows
Const Key_Application%  = Key_Apps
Const Key_App%          = Key_Apps

;-( Signs )----------------------------------------------;

Const Key_BackQuote%    = 41
Const Key_Minus%        = 12
Const Key_Equals%       = 13
Const Key_LeftBracket%  = 26
Const Key_RightBracket% = 27
Const Key_BackSlash%    = 43
Const Key_SemiColon%    = 39
Const Key_Quote%        = 40
Const Key_Comma%        = 51
Const Key_Period%       = 52
Const Key_Slash%        = 53

;-( Keypad controls )------------------------------------;

Const Key_PadSlash%    = 181
Const Key_PadAsterisk% =  55
Const Key_PadMinus%    =  74
Const Key_PadPlus%     =  78
Const Key_PadEnter%    = 156
Const Key_PadPeriod%   =  83

; alternative names
Const Key_PadStar%   = Key_PadAsterisk
Const Key_PadReturn% = Key_PadEnter

;-( Arrow keys )-----------------------------------------;

Const Key_CursorUp%    = 200
Const Key_CursorDown%  = 208
Const Key_CursorLeft%  = 203
Const Key_CursorRight% = 205

; alternative names
Const Key_ArrowUp%    = Key_CursorUp
Const Key_ArrowDown%  = Key_CursorDown
Const Key_ArrowLeft%  = Key_CursorLeft
Const Key_ArrowRight% = Key_CursorRight

Const Key_Up%    = Key_ArrowUp
Const Key_Down%  = Key_ArrowDown
Const Key_Left%  = Key_ArrowLeft
Const Key_Right% = Key_ArrowRight

;-( Special keys )---------------------------------------;

Const Key_Power% = 222
Const Key_Sleep% = 223
Const Key_Wake%  = 227

Const Key_MediaStop       = 149
Const Key_MediaPrevTrack  = 144
Const Key_MediaNextTrack  = 153
Const Key_MediaMute       = 160
Const Key_MediaPlayPause  = 162
Const Key_MediaQuit       = 164
Const Key_MediaVolumeDown = 174
Const Key_MediaVolumeUp   = 176
Const Key_MediaSelect     = 237

Const Key_WebHome      = 178
Const Key_WebSearch    = 229
Const Key_WebFavorites = 230
Const Key_WebRefresh   = 231
Const Key_WebStop      = 232
Const Key_WebForward   = 233
Const Key_WebBack      = 234

Const Key_Calculator = 161
Const Key_MyComputer = 235
Const Key_Mail       = 236

;-( Custom values )--------------------------------------;

Const Key_None    = 0
Const Key_Unknown = Key_MaxKeys-1
Const Key_Invalid = Key_MaxKeys

Const Key_NumLock_Light    = 1001
Const Key_CapsLock_Light   = 1002
Const Key_ScrollLock_Light = 1003
Const Key_InsertLock_Light = 1004

;======================================================================================================================;
;  Key names

Dim Key_Name$ ( Key_MaxKeys )

;-( Numerical )------------------------------------------;

Key_Name ( Key_1 ) = "1"
Key_Name ( Key_2 ) = "2"
Key_Name ( Key_3 ) = "3"
Key_Name ( Key_4 ) = "4"
Key_Name ( Key_5 ) = "5"
Key_Name ( Key_6 ) = "6"
Key_Name ( Key_7 ) = "7"
Key_Name ( Key_8 ) = "8"
Key_Name ( Key_9 ) = "9"
Key_Name ( Key_0 ) = "0"

;-( Alphabetic )-----------------------------------------;

Key_Name ( Key_Q ) = "Q"
Key_Name ( Key_W ) = "W"
Key_Name ( Key_E ) = "E"
Key_Name ( Key_R ) = "R"
Key_Name ( Key_T ) = "T"
Key_Name ( Key_Y ) = "Y"
Key_Name ( Key_U ) = "U"
Key_Name ( Key_I ) = "I"
Key_Name ( Key_O ) = "O"
Key_Name ( Key_P ) = "P"
Key_Name ( Key_A ) = "A"
Key_Name ( Key_S ) = "S"
Key_Name ( Key_D ) = "D"
Key_Name ( Key_F ) = "F"
Key_Name ( Key_G ) = "G"
Key_Name ( Key_H ) = "H"
Key_Name ( Key_J ) = "J"
Key_Name ( Key_K ) = "K"
Key_Name ( Key_L ) = "L"
Key_Name ( Key_Z ) = "Z"
Key_Name ( Key_X ) = "X"
Key_Name ( Key_C ) = "C"
Key_Name ( Key_V ) = "V"
Key_Name ( Key_B ) = "B"
Key_Name ( Key_N ) = "N"
Key_Name ( Key_M ) = "M"

;-( Numeric keypad )-------------------------------------;

Key_Name ( Key_Pad7 ) = "NumPad 7"
Key_Name ( Key_Pad8 ) = "NumPad 8"
Key_Name ( Key_Pad9 ) = "NumPad 9"
Key_Name ( Key_Pad4 ) = "NumPad 4"
Key_Name ( Key_Pad5 ) = "NumPad 5"
Key_Name ( Key_Pad6 ) = "NumPad 6"
Key_Name ( Key_Pad1 ) = "NumPad 1"
Key_Name ( Key_Pad2 ) = "NumPad 2"
Key_Name ( Key_Pad3 ) = "NumPad 3"
Key_Name ( Key_Pad0 ) = "NumPad 0"

;-( Function keys )--------------------------------------;

Key_Name ( Key_Function1  ) = "F1"
Key_Name ( Key_Function2  ) = "F2"
Key_Name ( Key_Function3  ) = "F3"
Key_Name ( Key_Function4  ) = "F4"
Key_Name ( Key_Function5  ) = "F5"
Key_Name ( Key_Function6  ) = "F6"
Key_Name ( Key_Function7  ) = "F7"
Key_Name ( Key_Function8  ) = "F8"
Key_Name ( Key_Function9  ) = "F9"
Key_Name ( Key_Function10 ) = "F10"
Key_Name ( Key_Function11 ) = "F11"
Key_Name ( Key_Function12 ) = "F12"

;-( Shiftstate )-----------------------------------------;

Key_Name ( Key_LeftShift  ) = "Left Shift"
Key_Name ( Key_RightShift ) = "Right Shift"
Key_Name ( Key_LeftAlt    ) = "Left Alt"
Key_Name ( Key_RightAlt   ) = "Right Alt"
Key_Name ( Key_LeftCtrl   ) = "Left Control"
Key_Name ( Key_RightCtrl  ) = "Right Control"
Key_Name ( Key_CapsLock   ) = "CapsLock"
Key_Name ( Key_NumLock    ) = "NumLock"
Key_Name ( Key_ScrollLock ) = "ScrollLock"

;-( Control keys )---------------------------------------;

Key_Name ( Key_Space        ) = "SpaceBar"
Key_Name ( Key_Enter        ) = "Enter"
Key_Name ( Key_Tab          ) = "Tab"
Key_Name ( Key_BackSpace    ) = "BackSpace"
Key_Name ( Key_Insert       ) = "Insert"
Key_Name ( Key_Delete       ) = "Delete"
Key_Name ( Key_Home         ) = "Home"
Key_Name ( Key_End          ) = "End"
Key_Name ( Key_PageUp       ) = "PageUp"
Key_Name ( Key_PageDown     ) = "PageDown"
Key_Name ( Key_Escape       ) = "Escape"
Key_Name ( Key_Pause        ) = "Pause"
Key_Name ( Key_PrintScreen  ) = "PrintScreen"
Key_Name ( Key_LeftWindows  ) = "Left Windows"
Key_Name ( Key_RightWindows ) = "Right Windows"
Key_Name ( Key_Apps         ) = "Apps"

;-( Signs )----------------------------------------------;

Key_Name ( Key_BackQuote    ) = "BackQuote `"
Key_Name ( Key_Minus        ) = "Minus -"
Key_Name ( Key_Equals       ) = "Equals ="
Key_Name ( Key_LeftBracket  ) = "Left Bracket ["
Key_Name ( Key_RightBracket ) = "Right Bracket ]"
Key_Name ( Key_BackSlash    ) = "BackSlash \"
Key_Name ( Key_SemiColon    ) = "SemiColon ;"
Key_Name ( Key_Quote        ) = "Quote '"
Key_Name ( Key_Comma        ) = "Comma ,"
Key_Name ( Key_Period       ) = "Period ."
Key_Name ( Key_Slash        ) = "Slash /"

;-( Keypad controls )------------------------------------;

Key_Name ( Key_PadSlash    ) = "NumPad Slash /"
Key_Name ( Key_PadAsterisk ) = "NumPad Asterisk *"
Key_Name ( Key_PadMinus    ) = "NumPad Minus -"
Key_Name ( Key_PadPlus     ) = "NumPad Plus +"
Key_Name ( Key_PadEnter    ) = "NumPad Enter"
Key_Name ( Key_PadPeriod   ) = "NumPad Period ."

;-( Cursors )--------------------------------------------;

;Key_Name ( Key_CursorUp    ) = "Cursor Up"
;Key_Name ( Key_CursorDown  ) = "Cursor Down"
;Key_Name ( Key_CursorLeft  ) = "Cursor Left"
;Key_Name ( Key_CursorRight ) = "Cursor Right"
Key_Name ( Key_CursorUp    ) = "Arrow Up"
Key_Name ( Key_CursorDown  ) = "Arrow Down"
Key_Name ( Key_CursorLeft  ) = "Arrow Left"
Key_Name ( Key_CursorRight ) = "Arrow Right"

;-( Special keys )---------------------------------------;

Key_Name ( Key_Power        ) = "Power"
Key_Name ( Key_Sleep        ) = "Sleep"
Key_Name ( Key_Wake         ) = "Wake"

Key_Name( Key_MediaStop       ) = "Media: Stop"
Key_Name( Key_MediaPrevTrack  ) = "Media: Previous Track"
Key_Name( Key_MediaNextTrack  ) = "Media: Next Track"
Key_Name( Key_MediaMute       ) = "Media: Mute"
Key_Name( Key_MediaPlayPause  ) = "Media: Play/Pause"
Key_Name( Key_MediaQuit       ) = "Media: Quit"
Key_Name( Key_MediaVolumeDown ) = "Media: Volume Down"
Key_Name( Key_MediaVolumeUp   ) = "Media: Volume Up"
Key_Name( Key_MediaSelect     ) = "Media: Select"

Key_Name( Key_WebHome      ) = "Web: Home"
Key_Name( Key_WebSearch    ) = "Web: Search"
Key_Name( Key_WebFavorites ) = "Web: Favorites"
Key_Name( Key_WebRefresh   ) = "Web: Refresh"
Key_Name( Key_WebStop      ) = "Web: Stop"
Key_Name( Key_WebForward   ) = "Web: Forward"
Key_Name( Key_WebBack      ) = "Web: Back"

Key_Name( Key_Calculator ) = "Calculator"
Key_Name( Key_MyComputer ) = "My Computer"
Key_Name( Key_Mail       ) = "Mail"

;-( Custom )---------------------------------------------;

Key_Name ( Key_None        ) = "None"
Key_Name ( Key_Unknown     ) = "Unknown"
Key_Name ( Key_Invalid     ) = "Invalid"

;,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,
;                                                        ;
;   Short names                                          ;
;                                                        ;
;--------------------------------------------------------'

Dim Key_ShortName$( Key_MaxKeys )

Key_ShortName( Key_1 ) = "1"
Key_ShortName( Key_2 ) = "2"
Key_ShortName( Key_3 ) = "3"
Key_ShortName( Key_4 ) = "4"
Key_ShortName( Key_5 ) = "5"
Key_ShortName( Key_6 ) = "6"
Key_ShortName( Key_7 ) = "7"
Key_ShortName( Key_8 ) = "8"
Key_ShortName( Key_9 ) = "9"
Key_ShortName( Key_0 ) = "0"
Key_ShortName( Key_Q ) = "Q"
Key_ShortName( Key_W ) = "W"
Key_ShortName( Key_E ) = "E"
Key_ShortName( Key_R ) = "R"
Key_ShortName( Key_T ) = "T"
Key_ShortName( Key_Y ) = "Y"
Key_ShortName( Key_U ) = "U"
Key_ShortName( Key_I ) = "I"
Key_ShortName( Key_O ) = "O"
Key_ShortName( Key_P ) = "P"
Key_ShortName( Key_A ) = "A"
Key_ShortName( Key_S ) = "S"
Key_ShortName( Key_D ) = "D"
Key_ShortName( Key_F ) = "F"
Key_ShortName( Key_G ) = "G"
Key_ShortName( Key_H ) = "H"
Key_ShortName( Key_J ) = "J"
Key_ShortName( Key_K ) = "K"
Key_ShortName( Key_L ) = "L"
Key_ShortName( Key_Z ) = "Z"
Key_ShortName( Key_X ) = "X"
Key_ShortName( Key_C ) = "C"
Key_ShortName( Key_V ) = "V"
Key_ShortName( Key_B ) = "B"
Key_ShortName( Key_N ) = "N"
Key_ShortName( Key_M ) = "M"
Key_ShortName( Key_Pad7 ) = "7"
Key_ShortName( Key_Pad8 ) = "8"
Key_ShortName( Key_Pad9 ) = "9"
Key_ShortName( Key_Pad4 ) = "4"
Key_ShortName( Key_Pad5 ) = "5"
Key_ShortName( Key_Pad6 ) = "6"
Key_ShortName( Key_Pad1 ) = "1"
Key_ShortName( Key_Pad2 ) = "2"
Key_ShortName( Key_Pad3 ) = "3"
Key_ShortName( Key_Pad0 ) = "0"
Key_ShortName( Key_Function1  ) = "F1"
Key_ShortName( Key_Function2  ) = "F2"
Key_ShortName( Key_Function3  ) = "F3"
Key_ShortName( Key_Function4  ) = "F4"
Key_ShortName( Key_Function5  ) = "F5"
Key_ShortName( Key_Function6  ) = "F6"
Key_ShortName( Key_Function7  ) = "F7"
Key_ShortName( Key_Function8  ) = "F8"
Key_ShortName( Key_Function9  ) = "F9"
Key_ShortName( Key_Function10 ) = "F10"
Key_ShortName( Key_Function11 ) = "F11"
Key_ShortName( Key_Function12 ) = "F12"
Key_ShortName( Key_LeftShift  ) = "Shift"
Key_ShortName( Key_RightShift ) = "Shift"
Key_ShortName( Key_LeftAlt    ) = "Alt"
Key_ShortName( Key_RightAlt   ) = "Alt"
Key_ShortName( Key_LeftCtrl   ) = "Ctrl"
Key_ShortName( Key_RightCtrl  ) = "Ctrl"
Key_ShortName( Key_CapsLock   ) = "Caps"
Key_ShortName( Key_NumLock    ) = "Num"
Key_ShortName( Key_ScrollLock ) = "ScrLk"
Key_ShortName( Key_Space        ) = "Space"
Key_ShortName( Key_Enter        ) = "Enter"
Key_ShortName( Key_Tab          ) = "Tab"
Key_ShortName( Key_BackSpace    ) = "BkSp"
Key_ShortName( Key_Insert       ) = "Ins"
Key_ShortName( Key_Delete       ) = "Del"
Key_ShortName( Key_Home         ) = "Home"
Key_ShortName( Key_End          ) = "End"
Key_ShortName( Key_PageUp       ) = "PgUp"
Key_ShortName( Key_PageDown     ) = "PgDn"
Key_ShortName( Key_Escape       ) = "Esc"
Key_ShortName( Key_Pause        ) = "Pause"
Key_ShortName( Key_PrintScreen  ) = "PrtSc"
Key_ShortName( Key_LeftWindows  ) = "Win"
Key_ShortName( Key_RightWindows ) = "Win"
Key_ShortName( Key_Apps         ) = "Apps"
Key_ShortName( Key_Power        ) = "Pow"
Key_ShortName( Key_Sleep        ) = "Slp"
Key_ShortName( Key_Wake         ) = "Wake"
Key_ShortName( Key_BackQuote    ) = "`"
Key_ShortName( Key_Minus        ) = "-"
Key_ShortName( Key_Equals       ) = "="
Key_ShortName( Key_LeftBracket  ) = "["
Key_ShortName( Key_RightBracket ) = "]"
Key_ShortName( Key_BackSlash    ) = "\"
Key_ShortName( Key_SemiColon    ) = ";"
Key_ShortName( Key_Quote        ) = "'"
Key_ShortName( Key_Comma        ) = ","
Key_ShortName( Key_Period       ) = "."
Key_ShortName( Key_Slash        ) = "/"
Key_ShortName( Key_PadSlash    ) = "/"
Key_ShortName( Key_PadAsterisk ) = "*"
Key_ShortName( Key_PadMinus    ) = "-"
Key_ShortName( Key_PadPlus     ) = "+"
Key_ShortName( Key_PadEnter    ) = "Enter"
Key_ShortName( Key_PadPeriod   ) = "."
Key_ShortName( Key_CursorUp    ) = "^";"/\"
Key_ShortName( Key_CursorDown  ) = "v";"\/"
Key_ShortName( Key_CursorLeft  ) = "<"
Key_ShortName( Key_CursorRight ) = ">"
Key_ShortName( Key_MediaStop       ) = "[]"
Key_ShortName( Key_MediaPrevTrack  ) = "<<"
Key_ShortName( Key_MediaNextTrack  ) = ">>"
Key_ShortName( Key_MediaMute       ) = "(o)"
Key_ShortName( Key_MediaPlayPause  ) = ">/||"
Key_ShortName( Key_MediaQuit       ) = "(|)"
Key_ShortName( Key_MediaVolumeDown ) = "\/"
Key_ShortName( Key_MediaVolumeUp   ) = "/\"
Key_ShortName( Key_MediaSelect     ) = "<>"
Key_ShortName( Key_WebHome      ) = "Home"
Key_ShortName( Key_WebSearch    ) = "Srch"
Key_ShortName( Key_WebFavorites ) = "Favs"
Key_ShortName( Key_WebRefresh   ) = "Refr"
Key_ShortName( Key_WebStop      ) = "Stop"
Key_ShortName( Key_WebForward   ) = "Fwd"
Key_ShortName( Key_WebBack      ) = "Back"
Key_ShortName( Key_Calculator ) = "Calc"
Key_ShortName( Key_MyComputer ) = "MyCmp"
Key_ShortName( Key_Mail       ) = "Mail"
Key_ShortName( Key_Unknown ) = "?"

;,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,
;                                                        ;
;   Visual keyboard keys (low and high chars)            ;
;                                                        ;
;--------------------------------------------------------'

Dim Key_VisualLowChar$( Key_MaxKeys )

Key_VisualLowChar( Key_1 ) = "1"
Key_VisualLowChar( Key_2 ) = "2"
Key_VisualLowChar( Key_3 ) = "3"
Key_VisualLowChar( Key_4 ) = "4"
Key_VisualLowChar( Key_5 ) = "5"
Key_VisualLowChar( Key_6 ) = "6"
Key_VisualLowChar( Key_7 ) = "7"
Key_VisualLowChar( Key_8 ) = "8"
Key_VisualLowChar( Key_9 ) = "9"
Key_VisualLowChar( Key_0 ) = "0"
Key_VisualLowChar( Key_BackQuote    ) = "`"
Key_VisualLowChar( Key_Minus        ) = "-"
Key_VisualLowChar( Key_Equals       ) = "="
Key_VisualLowChar( Key_LeftBracket  ) = "["
Key_VisualLowChar( Key_RightBracket ) = "]"
Key_VisualLowChar( Key_BackSlash    ) = "\"
Key_VisualLowChar( Key_SemiColon    ) = ";"
Key_VisualLowChar( Key_Quote        ) = "'"
Key_VisualLowChar( Key_Comma        ) = ","
Key_VisualLowChar( Key_Period       ) = "."
Key_VisualLowChar( Key_Slash        ) = "/"

Dim Key_VisualHighChar$( Key_MaxKeys )

Key_VisualHighChar( Key_1 ) = "!"
Key_VisualHighChar( Key_2 ) = "@"
Key_VisualHighChar( Key_3 ) = "#"
Key_VisualHighChar( Key_4 ) = "$"
Key_VisualHighChar( Key_5 ) = "%"
Key_VisualHighChar( Key_6 ) = "^"
Key_VisualHighChar( Key_7 ) = "&"
Key_VisualHighChar( Key_8 ) = "*"
Key_VisualHighChar( Key_9 ) = "("
Key_VisualHighChar( Key_0 ) = ")"
Key_VisualHighChar( Key_BackQuote    ) = "~"
Key_VisualHighChar( Key_Minus        ) = "_"
Key_VisualHighChar( Key_Equals       ) = "+"
Key_VisualHighChar( Key_LeftBracket  ) = "{"
Key_VisualHighChar( Key_RightBracket ) = "}"
Key_VisualHighChar( Key_BackSlash    ) = "|"
Key_VisualHighChar( Key_SemiColon    ) = ":"
Key_VisualHighChar( Key_Quote        ) = Chr(34)
Key_VisualHighChar( Key_Comma        ) = "<"
Key_VisualHighChar( Key_Period       ) = ">"
Key_VisualHighChar( Key_Slash        ) = "?"

;,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,
;                                                        ;
;   Keyboard shift-relative ASCII keys                   ;
;                                                        ;
;--------------------------------------------------------'

Global Key_VisualLowChars$  = "`1234567890-=[]\;',./"
Global Key_VisualHighChars$ = "~!@#$%^&*()_+{}|:"+Chr(34)+"<>?"
Global Key_InputLowChars$  = "`1234567890-=[]\;',./qwertyuiopasdfghjklzxcvbnm"
Global Key_InputHighChars$ = "~!@#$%^&*()_+{}|:"+Chr(34)+"<>?QWERTYUIOPASDFGHJKLZXCVBNM"
; convert a high (upper) ascii char to a low ascii char
Function Key_LowName$( char$ )
    ;; maybe already a lower char
    ;Local pos = Instr( Key_InputLowChars , char )
    ;If pos Then Return char
    ; check for the corresponding high char
    Local pos = Instr( Key_InputHighChars , char )
    If pos Then Return Mid( Key_InputHighChars , pos,1 )
    ; not in list -> invalidate (null)
    Return ""
End Function
; convert a low ascii char to a high (upper) ascii char
Function Key_HighName$( char$ )
    ;; maybe already a high char
    ;Local pos = Instr( Key_InputHighChars , char )
    ;If pos Then Return char
    ; check for the corresponding low char
    Local pos = Instr( Key_InputLowChars , char )
    If pos Then Return Mid( Key_InputLowChars , pos,1 )
    ; not in list -> invalidate (null)
    Return ""
End Function

;======================================================================================================================;

;~IDEal Editor Parameters:
;~C#Blitz3D198