
; Custom Print Library V3
; - contains both new and old systems
;   see reference and documentation at the bottom of this file
; - some parts of the new and old systems are difficult to use
;   modify to your pleasure
; - can introduce blitzprint.lib features in here to simplify
;   for example to override clscolor, and fit to screenheight

; Created by Teasy <TheChange@yahoo.com>
; Original release: 16 Aug 2002 at 0000

; 15 Jun 2007
; -----------
; * Added functions PrintSX and PrintSY to return screen position
;   rather than cursor position like PrintX and PrintY do.
; * Modified PrintText: swapped "\" with "|".
;   It seems more logical to have "\" for linefeed and/or omitting char
;   and using "|" to print without linefeed (since "|" looks like a cursor)

;------------------------------------------------------------------------------

Const Print_Cursor_BlinkingRate_Default = 100  ; Number of millisecs between state changes of small blinking cursor

Type Print_Environment
    ;- System 1
    Field CursorX
    Field CursorY
    Field TextRGB
    Field CursorRGB
    Field ScreenSizeX
    Field ScreenSizeY
    Field BehindCursor  ; Image
    ;- System 2
    Field Cursor_Image      ; To remember what's behind the small blinking cursor
    Field Cursor_Display    ; Whether to display the cursor at all
    Field Cursor_State      ; On or off, changing at the blinking rate
    Field Cursor_RememberX  ; Remember position to remove traces of old cursor when moved
    Field Cursor_RememberY
    Field Cursor_LastStateChange  ; Last time the cursor changed state (on or off)
    Field Cursor_BlinkingRate
    ;- System 3
    Field FontSizeX
    Field FontSizeY
    Field CharsX
    Field CharsY
End Type

Global Print_Environment.Print_Environment

Function Print_Environment_Initialize ()
    If Print_Environment <> Null
        Print_Environment_Destroy Print_Environment
    End If
    Print_Environment = Print_Environment_Create ()
End Function

Function Print_Environment_Create.Print_Environment ()
    Local Print_Environment.Print_Environment

    Print_Environment = New Print_Environment
    Print_Environment\TextRGB = 32 Shl 16 + 192 Shl 8 + 32
    Print_Environment\CursorRGB = Print_Environment\TextRGB
    Print_Environment\ScreenSizeX = GraphicsWidth ()
    Print_Environment\ScreenSizeY = GraphicsHeight ()
    Print_Environment\FontSizeX = FontWidth ()
    Print_Environment\FontSizeY = FontHeight ()
    Print_Environment\CharsX = Print_Environment\ScreenSizeX / Print_Environment\FontSizeX
    Print_Environment\CharsY = Print_Environment\ScreenSizeY / Print_Environment\FontSizeY
    Print_Environment\Cursor_BlinkingRate = Print_Cursor_BlinkingRate_Default

    Print_Environment\BehindCursor = CreateImage ( Print_Environment\FontSizeX , Print_Environment\FontSizeY )
    MaskImage Print_Environment\BehindCursor , 255 , 0 , 250
    Print_Environment\Cursor_Image = CreateImage ( Print_Environment\FontSizeX , 2 )
    MaskImage Print_Environment\Cursor_Image , 255 , 0 , 250  ; Make sure that black is not transparent

    Return Print_Environment
End Function

Function Print_Environment_Destroy ( Print_Environment.Print_Environment )
    FreeImage Print_Environment\BehindCursor
    FreeImage Print_Environment\Cursor_Image
    Delete Print_Environment
End Function

Function PrintLocate( X = -1 , Y = -1 )
    If X >= 0 Then Print_Environment\CursorX = X * Print_Environment\FontSizeX
    If Y >= 0 Then Print_Environment\CursorY = Y * Print_Environment\FontSizeY
    Print_Environment_Limit_Cursor
End Function

Function PrintHome ()
    Print_Environment\CursorX = 0
    Print_Environment\CursorY = 0
End Function

Function PrintUp ()
    Print_Environment\CursorY = Print_Environment\CursorY - Print_Environment\FontSizeY
    If Print_Environment\CursorY < 0
        Print_Environment\CursorY = 0
    EndIf
End Function

Function PrintDown ()  ; 1 bug (no fit to screen height, see blitzprint.lib for fix)
    Print_Environment\CursorY = Print_Environment\CursorY + Print_Environment\FontSizeY
    If Print_Environment\CursorY >= (Print_Environment\CharsY) * Print_Environment\FontSizeY
        Print_Environment\CursorY = (Print_Environment\CharsY-1) * Print_Environment\FontSizeY
        PrintScroll
    EndIf
End Function

Function PrintLeft ()
    Print_Environment\CursorX = Print_Environment\CursorX - Print_Environment\FontSizeX
    If Print_Environment\CursorX < 0
        Print_Environment\CursorX = 0
    EndIf
End Function

Function PrintRight ()
    Print_Environment\CursorX = Print_Environment\CursorX + Print_Environment\FontSizeX
    If Print_Environment\CursorX >= (Print_Environment\CharsX) * Print_Environment\FontSizeX
        Print_Environment\CursorX = 0
        PrintDown
    EndIf
End Function

Function PrintText ( Text$ = "" )
    Local PrintText$
    Local MovePos
    Local LineFeed

    If Right ( Text , 1 ) = "|"  ; like Write
        MovePos = True
        LineFeed = False
        PrintText = Left ( Text , Len ( Text ) - 1 )
    ElseIf Right ( Text , 1 ) = "/"  ; cursor stay
        MovePos = False
        LineFeed = False
        PrintText = Left ( Text , Len ( Text ) - 1 )
    Else
        MovePos = True
        LineFeed = True
        If Right ( Text , 1 ) = "\"  ; linefeed/normal, like Print
            PrintText = Left ( Text , Len ( Text ) - 1 )
        Else
            PrintText = Text
        EndIf
    EndIf

    Color 0 , 0 , Print_Environment\TextRGB
    Text Print_Environment\CursorX , Print_Environment\CursorY , PrintText

    If MovePos
        Print_Environment\CursorX = Print_Environment\CursorX + Print_Environment\FontSizeX * Len ( PrintText )
    EndIf
    If Print_Environment\CursorX >= Print_Environment\CharsX * Print_Environment\FontSizeX
        Print_Environment\CursorX = 0
    EndIf
    If LineFeed
        Print_Environment\CursorX = 0
        PrintDown
    EndIf
End Function

Function PrintScroll () ; 2 bugs
    CopyRect 0 , Print_Environment\FontSizeY , Print_Environment\ScreenSizeX , Print_Environment\ScreenSizeY - Print_Environment\FontSizeY , 0 , 0
    Color 0 , 0 , 0  ; Naughty
    Rect 0 , Print_Environment\ScreenSizeY - Print_Environment\FontSizeY , Print_Environment\ScreenSizeX , Print_Environment\FontSizeY , True
End Function

Function PrintLineFeed ()
    PrintDown
    Print_Environment\CursorX = 0
End Function

Function PrintBackSpace () ; 1 bug
    PrintLeft
    Color 0 , 0 , 0  ; Naughty
    Rect Print_Environment\CursorX , Print_Environment\CursorY , Print_Environment\FontSizeX , Print_Environment\FontSizeY , True
End Function

Function PrintClear ( Chars% = 1 ) ; 1 bug
    Color 0 , 0 , 0  ; Naughty
    Rect Print_Environment\CursorX , Print_Environment\CursorY , Print_Environment\FontSizeX * Chars , Print_Environment\FontSizeY , True
End Function

Function PrintX% ()
    Return Print_Environment\CursorX / Print_Environment\FontSizeX
End Function

Function PrintY% ()
    Return Print_Environment\CursorY / Print_Environment\FontSizeY
End Function

Function PrintSX% ()
    Return Print_Environment\CursorX
End Function

Function PrintSY% ()
    Return Print_Environment\CursorY
End Function

Function PrintShowCursor ( Invert% = False )
    Local SourceBuffer
    Local DestBuffer

    If Not Invert
        SourceBuffer = 0
        DestBuffer = ImageBuffer ( Print_Environment\BehindCursor )
        CopyRect Print_Environment\CursorX , Print_Environment\CursorY , Print_Environment\FontSizeX , Print_Environment\FontSizeY , 0 , 0 , SourceBuffer , DestBuffer
        Color 0 , 0 , Print_Environment\CursorRGB
        Rect Print_Environment\CursorX , Print_Environment\CursorY , Print_Environment\FontSizeX , Print_Environment\FontSizeY , True
    Else
        PrintInvertRect Print_Environment\CursorX , Print_Environment\CursorY , Print_Environment\FontSizeX , Print_Environment\FontSizeY
    EndIf
End Function

Function PrintHideCursor ( Invert% = False )
    If Not Invert
        DrawBlock Print_Environment\BehindCursor , Print_Environment\CursorX , Print_Environment\CursorY
    Else
        PrintInvertRect Print_Environment\CursorX , Print_Environment\CursorY , Print_Environment\FontSizeX , Print_Environment\FontSizeY
    EndIf
End Function

Function PrintInvertRect ( PosX% , PosY% , SizeX% , SizeY% )
    Local DestX
    Local DestY
    Local RGB
    Local R
    Local G
    Local B
    Local X
    Local Y

    DestX = PosX + SizeX
    DestY = PosY + SizeY
    LockBuffer
    For X = PosX To DestX
        For Y = PosY To DestY
            RGB = ReadPixelFast ( X , Y ) And $FFFFFF
            R = 255 - RGB Shr 16 And 255
            G = 255 - RGB Shr 8 And 255
            B = 255 - RGB And 255
            WritePixelFast X , Y , R Shl 16 + G Shl 8 + B
        Next
    Next
    UnlockBuffer
End Function

Function PrintTextColor ( Red% , Green% , Blue% )
    Print_Environment\TextRGB = Red Shl 16 + Green Shl 8 + Blue
End Function

Function PrintCursorColor ( Red% , Green% , Blue% )
    Print_Environment\CursorRGB = Red Shl 16 + Green Shl 8 + Blue
End Function

Function Print_Environment_Cursor_Display_Toggle ( Display% = -1 )
    If Display = -1
      Print_Environment\Cursor_Display = Not Print_Environment\Cursor_Display
    Else
      Print_Environment\Cursor_Display = Display
    End If
End Function

Function Print_Environment_Cursor_Display_Update () ; Note: Uses the current buffer for all operations.
    If Print_Environment\Cursor_Display
        If MilliSecs () > Print_Environment\Cursor_LastStateChange + Print_Environment\Cursor_BlinkingRate
            Print_Environment\Cursor_LastStateChange = MilliSecs ()
            Print_Environment\Cursor_State = Not Print_Environment\Cursor_State
            If Print_Environment\Cursor_State
                CopyRect Print_Environment\CursorX , Print_Environment\CursorY + Print_Environment\FontSizeY - 2 , Print_Environment\FontSizeX , 2 , 0 , 0 , 0 , ImageBuffer ( Print_Environment\Cursor_Image )
                Color 0 , 0 , Print_Environment\CursorRGB
                Rect Print_Environment\CursorX , Print_Environment\CursorY + Print_Environment\FontSizeY - 2 , Print_Environment\FontSizeX , 2 , True
            Else
                DrawImage Print_Environment\Cursor_Image , Print_Environment\CursorX , Print_Environment\CursorY + Print_Environment\FontSizeY - 2
            End If
        End If
    End If
End Function

Function Print_Environment_Cursor_Vanish ()
    If Print_Environment\Cursor_Display
        If Print_Environment\Cursor_State
            DrawImage Print_Environment\Cursor_Image , Print_Environment\Cursor_RememberX , Print_Environment\Cursor_RememberY + Print_Environment\FontSizeY - 2
        End If
    End If
End Function

Function Print_Environment_Cursor_Learn ()
    If Print_Environment\Cursor_Display
        If Print_Environment\Cursor_RememberX <> Print_Environment\CursorX Or Print_Environment\Cursor_RememberY <> Print_Environment\CursorY
            Print_Environment\Cursor_RememberX = Print_Environment\CursorX
            Print_Environment\Cursor_RememberY = Print_Environment\CursorY
            CopyRect Print_Environment\CursorX , Print_Environment\CursorY + Print_Environment\FontSizeY - 2 , Print_Environment\FontSizeX , 2 , 0 , 0 , 0 , ImageBuffer ( Print_Environment\Cursor_Image )
        End If
    End If
End Function

Function Print_Environment_Cursor_Appear ()
    If Print_Environment\Cursor_Display
        If Print_Environment\Cursor_State
            Color 0 , 0 , Print_Environment\CursorRGB
            Rect Print_Environment\CursorX , Print_Environment\CursorY + Print_Environment\FontSizeY - 2 , Print_Environment\FontSizeX , 2 , True
        End If
    End If
End Function

Function Print_Environment_Limit_Cursor ()
    If Print_Environment\CursorX < 0 Then Print_Environment\CursorX = 0
    If Print_Environment\CursorY < 0 Then print_Environment\CursorY = 0
    If Print_Environment\CursorX >= Print_Environment\CharsX * Print_Environment\FontSizeX Then Print_Environment\CursorX = Print_Environment\CharsX * Print_Environment\FontSizeX
    If Print_Environment\CursorY >= Print_Environment\CharsY * Print_Environment\FontSizeY Then Print_Environment\CursorY = Print_Environment\CharsY * Print_Environment\FontSizeY
End Function

;----------------------------------------------------------------------------------------------------------------------;
;   Quick reference
;----------------------------------------------------------------------------------------------------------------------;
;
;   PrintHome () moves the Print Cursor to the upperleft corner of the screen
;   PrintUp () moves the Print Cursor one character up
;   PrintDown () moves the Print Cursor one character down
;   PrintLeft () moves the Print Cursor one character left
;   PrintRight () moves the Print Cursor one character right
;   PrintText ( Text$ = "" ) prints any line of text at the current cursor position ("\"=append,"/"=nomove,"|"=normal)
;   PrintScroll () scrolls the screen up one line of characters
;   PrintLineFeed () prints/appends a linefeed (same as PrintText without parameters)
;   PrintBackSpace () removes the character space in front of the cursor and moves the cursor back one place
;   PrintClear ( Chars% = 1 ) overwrites any number of characters at the cursor position without moving the cursor
;   PrintX% () returns the current horizontal cursor position
;   PrintY% () returns the current vertical cursor position
;   PrintShowCursor ( Invert% = False ) displays an overlapping cursor at the current cursor position
;   PrintHideCursor ( Invert% = False ) removes the overlapping cursor at the current cursor position
;   PrintInvertRect ( PosX% , PosY% , SizeX% , SizeY% ) inverts the colors of any rectangular area of the screen
;   PrintCursorColor ( Red% , Green% , Blue% ) sets the default color for the non-inverted and small blinking cursor
;   Print_Environment_Cursor_Display_Toggle ( Display% ) toggles the small blinking cursor to be displayed or not
;   Print_Environment_Cursor_Display_Update () displays and updates position & state of small blinking cursor on screen
;
;----------------------------------------------------------------------------------------------------------------------;

;----------------------------------------------------------------------------------------------------------------------;
; Documentation (Internal functions excluded)
; Note: some information contained here is outdated and/or incorrect.
;----------------------------------------------------------------------------------------------------------------------;
;   Function PrintHome ()
;     Function:  Moves the Print Cursor to the upperleft corner of the screen.
;     Parameters:  None.
;     Returns:  Nothing.
;     Example:  PrintHome
;----------------------------------------------------------------------------------------------------------------------;
;   Function PrintUp ()
;     Function:  Moves the Print Cursor one character up.
;     Parameters:  None.
;     Returns:  Nothing.
;     Example:  PrintUp
;----------------------------------------------------------------------------------------------------------------------;
;   Function PrintDown ()
;     Function:  Moves the Print Cursor one character down.
;     Parameters:  None.
;     Returns:  Nothing.
;     Example:  PrintDown
;----------------------------------------------------------------------------------------------------------------------;
;   Function PrintLeft ()
;     Function:  Moves the Print Cursor one character left.
;     Parameters:  None.
;     Returns:  Nothing.
;     Example:  PrintLeft
;----------------------------------------------------------------------------------------------------------------------;
;   Function PrintRight ()
;     Function:  Moves the Print Cursor one character right.
;     Parameters:  None.
;     Returns:  Nothing.
;     Example:  PrintRight
;----------------------------------------------------------------------------------------------------------------------;
;   Function PrintText ( Text$ = "" )
;     Function:  Prints any line of text at the current cursor position.
;     Parameters:
;       Text string (optional).
;         The line of text to use for display. Normally the text will be printed followed by a linefeed.
;         However there are 3 unique trailing characters serving special functions:
;          - Backslash     "\": Omit the linefeed. All text will be appended.
;          - Forward slash "/": Prevent the cursor from moving. Whatever you print, the cursor will not move.
;          - Pipe          "|": Normal printing. Use this if you want to print any of these special characters on
;                               screen last, as these special characters will be stripped from the output when last.
;         Omitting the text string will simply print a linefeed.
;         Note: The text being printed always has a transparent background.
;     Returns:  Nothing.
;     Examples:
;       PrintText       ; Print/append a linefeed.
;       PrintText "\|"  ; Print "\" with a linefeed.
;       PrintText "/|"  ; Print "/" with a linefeed.
;       PrintText "||"  ; Print "|" with a linefeed.
;       PrintText "x\"  ; Print "x" without a linefeed.
;       PrintText "x/"  ; Print "x" without moving the cursor.
;       PrintText "x|"  ; Same as PrintText "x".
;----------------------------------------------------------------------------------------------------------------------;
;   Function PrintScroll ()
;     Function:  Scrolls the screen up one line of characters.
;     Parameters:  None.
;     Returns:  Nothing.
;     Example:  PrintScroll
;----------------------------------------------------------------------------------------------------------------------;
;   Function PrintLineFeed ()
;     Function:  Prints/appends a linefeed. The same as PrintText without parameters.
;     Parameters:  None.
;     Returns:  Nothing.
;     Example:  PrintLineFeed
;----------------------------------------------------------------------------------------------------------------------;
;   Function PrintBackSpace ()
;     Function:  Removes the character space in front of the cursor and moves the cursor back one place.
;     Parameters:  None.
;     Returns:  Nothing.
;     Example:  PrintBackSpace
;----------------------------------------------------------------------------------------------------------------------;
;   Function PrintClear ( Chars% = 1 )
;     Function:  Prints a specified number of (overwriting) spaces at the cursor position without moving the cursor.
;     Parameters:
;       Number of characters.
;         This is the number of overwriting spaces. The default number of characters is 1, when omitting this parameter.
;     Returns:  Nothing.
;     Examples:
;       PrintClear               ; Clears one character space at the cursor.
;       PrintClear 10            ; Clears ten character spaces at the cursor.
;       PrintClear Len ( Text )  ; Clears the number of characters in the specified text string.
;----------------------------------------------------------------------------------------------------------------------;
;   Function PrintX% ()
;     Function:  Returns the current horizontal cursor position.
;     Parameters:  None.
;     Returns:  The current horizontal (X) cursor position.
;     Example:  RememberX = PrintX ()
;----------------------------------------------------------------------------------------------------------------------;
;   Function PrintY% ()
;     Function:  Returns the current vertical cursor position.
;     Parameters:  None.
;     Returns:  The current vertical (Y) cursor position.
;     Example:  RememberY = PrintY ()
;----------------------------------------------------------------------------------------------------------------------;
;   Function PrintShowCursor ( Invert% = False )
;     Function:
;       Displays an overlapping cursor at the current cursor position. The permanently drawn cursor will stay at the
;       drawn location until you call PrintHideCursor at the same cursor position. See the PrintHideCursor command for
;       more information and an example displaying these commands in action. Do not let the cursor go beyond the
;       dimensions of the screen when using this command, which normally cannot happen anyway.
;     Parameters:
;       Invert or overwrite cursor.
;         Specifying a True value as parameter will invert the color of the background to display the cursor.
;         Specifying a False value or omitting the parameter will result in an overlapping cursor in the current Print
;         Color.
;     Returns:  Nothing.
;     Example:  See the PrintHideCursor example.
;----------------------------------------------------------------------------------------------------------------------;
;   Function PrintHideCursor ( Invert% = False )
;     Function:
;       Removes the overlapping cursor at the current cursor position. Performing this command on a location where no
;       visible cursor resides can lead to the display of multiple simultaneous visible cursors. Also do not let the
;       cursor go beyond the dimensions of the screen when using this command, which normally cannot happen anyway.
;     Parameters:
;       Invert or overwrite cursor.
;         Specifying a True value as parameter will invert the color of background back to the original color.
;         Specifying a False value or omitting the parameter will remove the overlapping cursor at the current cursor
;         position. A temporary image is used to remember what was under the cursor.
;     Returns:  Nothing.
;     Example:
;       ;:. Tiny text editor .:; [ Using the old system ]
;       Graphics 640 , 480
;       PrintReset
;       PrintShowCursor True             ; Draw a visible cursor before editing.
;       Repeat
;         Key = GetKey ()
;         If Key
;           PrintHideCursor True         ; Hide the visible cursor before moving the print cursor.
;           PrintText Chr ( Key ) + "/"
;           PrintRight
;           PrintShowCursor True         ; Show the visible cursor after moving the print cursor.
;         End If
;       Until Key = 27
;       End
;----------------------------------------------------------------------------------------------------------------------;
;   Function PrintInvertRect ( PosX% , PosY% , SizeX% , SizeY% )
;     Function:
;       This is the function used to display the inverted cursor. You can use this function yourself to invert a
;       specific part of the current drawing buffer. As this function is very slow, like most pixel operations, do not
;       apply it to large portions of the screen. Be careful with this function, as it can crash your program when
;       drawing beyond the dimensions of the screen.
;     Parameters:
;       PosX - The upperleft horizontal (X) coordinate of the rectangular area to invert.
;       PosY - The upperleft vertical (Y) coordinate of the rectangular area to invert.
;       SizeX - The horizontal (X) span or size of the regular area to invert.
;       SizeY - The vertical (Y) span or size of the regular area to invert.
;     Returns:  Nothing
;     Example:
;       PrintInvertRect 0 , 0 , GraphicsWidth () , GraphicsHeight ()          ; Inverts the whole screen.
;       PrintInvertRect PrintX () , PrintY () , FontWidth () , FontHeight ()  ; Manually inverting a cursor.
;----------------------------------------------------------------------------------------------------------------------;
;   Function PrintCursorColor ( Red% , Green% , Blue% )
;     Function:  Sets the default color for the visible (non-inverted and small blinking) cursor.
;     Parameters:
;       The parameters used are exactly the same as those for the Blitz native Color command:
;       Red   - The Red color component.
;       Green - The Green color component.
;       Blue  - The Blue color component.
;       Together these color components make a combined RGB color.
;     Returns:  Nothing.
;     Example:
;       PrintColor 255 ,   0 ,   0  ; Red.
;       PrintColor   0 , 255 ,   0  ; Green.
;       PrintColor   0 ,   0 , 255  ; Blue.
;       PrintColor 255 , 255 , 255  ; White.
;       PrintColor 127 , 127 , 127  ; Grey.
;       PrintColor 255 , 255 ,   0  ; Yellow.
;       PrintColor   0 , 255 , 255  ; Cyan.
;       PrintColor 255 ,   0 , 255  ; Purple.
;       PrintColor 255 , 127 ,   0  ; Orange.
;       PrintColor   0 , 223 , 159  ; Creme green.
;       PrintColor   0 , 191 , 255  ; Creme blue.
;----------------------------------------------------------------------------------------------------------------------;
; Functions for the new system are not included in the documentation.
;----------------------------------------------------------------------------------------------------------------------;
