;-------------------------------------------------------------------------------
; * Generic rendering routines
;-------------------------------------------------------------------------------
; 2012-08-06
; - Added LimitTextWidth
; - Added WordWrappedText
; 2011-10-17
; - Added TextNoOutline
;-------------------------------------------------------------------------------


;===============================================================================
;
;   Rendering routines
;
;===============================================================================


;-------------------------------------------------------------------------------
; Draw (optional) colorized text with (optional) outline
;-------------------------------------------------------------------------------
Function ColorText( x,y , Text$ , CenterX=0,CenterY=0 , outr=-1,outg=-1,outb=-1 , separation=1 , br#=1.0,bg#=1.0,bb#=1.0 )
;-------------------------------------------------------------------------------
; * Sep 2006
; - Supports `rrrgggbbb
; - Added CenterX,CenterY (0,1,2)
; - Takes into account rgb command
; - Current color is recalled when no default color is set
; - Added variable width font support by stripping rgb command
; - Added outr=-x value for using rgb/x as outline color (goes for each outr,outg,outb)
;   for example -2 means use half rgb for outline color
; * Oct 2006
; - Added br,bg,bb: global brightness (0..1)
; - Optimizing: Draw per color section (skip to next `)
;   * Algorithm
;       PrevCharNum = 1
;       Repeat
;           CharNum = Instr( Text , "`" , PrevCharNum )
;           ; found a next color header
;           If CharNum > 0
;               ; grab (actual) text inbetween previous and next color header
;               Section = Mid( Text , PrevCharNum , CharNum-PrevCharNum )  ;"`123456789abc`123456789" 11,14->11,3
;               * DRAW (with offset advance)
;               * COLOR (with string parsing)
;               CharNum = CharNum + 3+3+3+1  ; skip over `rrrgggbbb at next char
;               PrevCharNum = CharNum
;           ; didn't find a next color header
;           Else
;               ; still something left to draw (tail)
;               If PrevCharNum <= Length
;                   Section = Mid( Text , PrevCharNum , Length-PrevCharNum+1 )  ;"abc`123456789abc" 14,16->14,3
;                   * DRAW
;               EndIf
;               ; all done
;               Exit
;           EndIf
;       Forever
; - Fixed original color ability
;   you can write something without color header, using current color, e.g. outside of function
;-------------------------------------------------------------------------------
    Local CharNum,Char$,Length
    Local r,g,b,ox,oxb,oy
    Local Raw$,outdiv
    Local Section$,PrevCharNum
    Local startr,startg,startb
    
    ; precalc length and set default color
    Length = Len( Text )
    startr = ColorRed() * br
    startg = ColorGreen() * bg
    startb = ColorBlue() * bb
    
    ; Strip `rrrgggbbb
    PrevCharNum = 1
    Repeat
        CharNum = Instr( Text , "`" , PrevCharNum )
        ; found a next color header
        If CharNum > 0
            ; grab (actual) text inbetween previous and next color header
            Raw = Raw + Mid( Text , PrevCharNum , CharNum-PrevCharNum )
            CharNum = CharNum + 3+3+3+1  ; skip over `rrrgggbbb at next char
            PrevCharNum = CharNum
        ; didn't find a next color header
        Else
            ; still something left to draw (tail)
            If PrevCharNum <= Length
                Raw = Raw + Mid( Text , PrevCharNum , Length-PrevCharNum+1 )
            EndIf
            ; all done
            Exit
        EndIf
    Forever
    
    ; CenterX: 0 = Left justify, 1 = Center, 2 = Right justify
    If CenterX = 2
        ox = -StringWidth( Raw )
    ElseIf CenterX = 1
        ox = -StringWidth( Raw ) / 2
    Else
        ox = 0
    EndIf
    ; CenterY: 0 = Top justify, 1 = Center, 2 = Bottom justify
    If CenterY = 2
        oy = -StringHeight( Raw )
    ElseIf CenterY = 1
        oy = -StringHeight( Raw ) / 2
    Else
        oy = 0
    EndIf
    
    ; Format and render outline (if used)
    If outr <> -1
        ; use default color
        r = startr
        g = startg
        b = startb
        ; offset x background (increased per section)
        oxb = ox
        PrevCharNum = 1
        Repeat
            CharNum = Instr( Text , "`" , PrevCharNum )
            ; found a next color header
            If CharNum > 0
                ; grab (actual) text inbetween previous and next color header
                ; "`123456789abc`123456789" 11,14->11,3
                Section = Mid( Text , PrevCharNum , CharNum-PrevCharNum )
                ; specific color
                If outr > -1
                    ; Draw outline only (if specified)
                    TextSpread oxb+x,oy+y , Section , False,False , outr*br,outg*bg,outb*bb , separation
                ; division of current color
                ElseIf outr < -1
                    ; Draw outline only (using rgb/-x)
                    ;TextSpread oxb+x,oy+y , Section , False,False , (r/Float -outr)*br,(g/Float -outg)*bg,(b/Float -outb)*bb , separation
                    TextSpread oxb+x,oy+y , Section , False,False , (r/-outr)*br,(g/-outg)*bg,(b/-outb)*bb , separation
                EndIf
                oxb = oxb + StringWidth( Section )
                If outr < -1
                    r = Mid( Text , CharNum+1,3 )  ; `rrr000000
                    g = Mid( Text , CharNum+4,3 )  ; `000ggg000
                    b = Mid( Text , CharNum+7,3 )  ; `000000bbb
                    Color r*br,g*bg,b*bb
                EndIf
                ; skip over `rrrgggbbb at next char
                CharNum = CharNum + 3+3+3+1
                PrevCharNum = CharNum
            ; didn't find a next color header
            Else
                ; still something left to draw (tail)
                If PrevCharNum <= Length
                    ;"abc`123456789abc" 14,16->14,3
                    Section = Mid( Text , PrevCharNum , Length-PrevCharNum+1 )  
                    If outr > -1
                        ; Draw outline only (if specified)
                        TextSpread oxb+x,oy+y , Section , False,False , outr*br,outg*bg,outb*bb , separation
                    ElseIf outr < -1
                        ; Draw outline only (using rgb/-x)
                        ;TextSpread oxb+x,oy+y , Section , False,False , (r/Float -outr)*br,(g/Float -outg)*bg,(b/Float -outb)*bb , separation
                        TextSpread oxb+x,oy+y , Section , False,False , (r/-outr)*br,(g/-outg)*bg,(b/-outb)*bb , separation
                    EndIf
                EndIf
                ; all done
                Exit
            EndIf
        Forever
    EndIf
    ; use default color
    r = startr
    g = startg
    b = startb
    Color r,g,b
    
    ; Format and render overlay (front)
    PrevCharNum = 1
    Repeat
        CharNum = Instr( Text , "`" , PrevCharNum )
        ; found a next color header
        If CharNum > 0
            ; grab (actual) text inbetween previous and next color header
            ; "`123456789abc`123456789" 11,14->11,3
            Section = Mid( Text , PrevCharNum , CharNum-PrevCharNum )
            Text ox+x,oy+y , Section
            ox = ox + StringWidth( Section )
            r = Mid( Text , CharNum+1,3 )  ; `rrr000000
            g = Mid( Text , CharNum+4,3 )  ; `000ggg000
            b = Mid( Text , CharNum+7,3 )  ; `000000bbb
            Color r*br,g*bg,b*bb
            ; skip over `rrrgggbbb at next char
            CharNum = CharNum + 3+3+3+1
            PrevCharNum = CharNum
        ; didn't find a next color header
        Else
            ; still something left to draw (tail)
            If PrevCharNum <= Length
                ; "abc`123456789abc" 14,16->14,3
                Section = Mid( Text , PrevCharNum , Length-PrevCharNum+1 )
                Text ox+x,oy+y , Section
            EndIf
            ; all done
            Exit
        EndIf
    Forever
;-------------------------------------------------------------------------------
End Function
;-------------------------------------------------------------------------------


; Like Text() only with fixed width characters
Function CharText( x,y , msg$ , cx=False,cy=False , r=-1,g=-1,b=-1 , ccw=-1,cch=-1 )
; ccw,cch: (optional) custom char width/height
    Local cw,ch
    Local length,char

    If ccw = -1
        cw = StringWidth( "A" )
    Else
        cw = ccw
    EndIf
    If cch = -1
        ch = StringHeight( "A" )
    Else
        ch = cch
    EndIf

    length = Len( msg )

    ; draw all chars from middle of char
    x = x + cw / 2
    y = y + ch / 2
    ; CenterX: 0 = Left justify, 1 = Center, 2 = Right justify
    If cx = 2
        x = x - cw * length;StringWidth( msg )
    ElseIf cx = 1
        x = x - cw * length / 2;StringWidth( msg ) / 2
    EndIf
    ; CenterY: 0 = Top justify, 1 = Center, 2 = Bottom justify
    If cy = 2
        y = y - ch;StringHeight( msg )
    ElseIf cy = 1
        y = y - ch / 2;StringHeight( msg ) / 2
    EndIf
    If b > -1
        Color r,g,b
    EndIf
    For char = 1 To length
        Text x,y , Mid( msg , char,1 ) , True,True
        x = x + cw
    Next
End Function


; Like Text() only with 3x3 outline, for example to make white text visible on white background
Function TextOutLine( PosX%,PosY% , Message$ , CenterX=False,CenterY=False , InR=255,InG=255,InB=255 , OutR=0,OutG=0,OutB=0 , Separation=1 )
; 09 sep 2007:
; - swapped inner/outer color (inner color is now white, outer is black)
    Local X%
    Local Y%
    Color OutR,OutG,OutB
    ; CenterX: 0 = Left justify, 1 = Center, 2 = Right justify
    If CenterX = 2
        PosX = PosX - StringWidth( Message )
        CenterX = False
    EndIf
    ; CenterY: 0 = Top justify, 1 = Center, 2 = Bottom justify
    If CenterY = 2
        PosY = PosY - StringHeight( Message )
        CenterY = False
    EndIf
    For X = -1 To 1
        For Y = -1 To 1
            Text PosX + X * Separation , PosY + Y * Separation , Message , CenterX , CenterY
        Next
    Next
    Color InR,InG,InB
    Text PosX,PosY , Message , CenterX,CenterY
End Function


; Like TextOutline() only with 1 color
Function TextSpread( PosX%,PosY% , Message$ , CenterX=False,CenterY=False , R=255,G=0,B=0 , Separation=1 )
    Local X%
    Local Y%
    Color R,G,B
    ; CenterX: 0 = Left justify, 1 = Center, 2 = Right justify
    If CenterX = 2
        PosX = PosX - StringWidth( Message )
        CenterX = False
    EndIf
    ; CenterY: 0 = Top justify, 1 = Center, 2 = Bottom justify
    If CenterY = 2
        PosY = PosY - StringHeight( Message )
        CenterY = False
    EndIf
    For X = -1 To 1
        For Y = -1 To 1
            Text PosX + X * Separation , PosY + Y * Separation , Message , CenterX , CenterY
        Next
    Next
End Function


; Like TextOutline() but without outline
Function TextNoOutLine( PosX,PosY , Message$ , CenterX=False,CenterY=False , InR=255,InG=255,InB=255 )
    If CenterX = 2
        PosX = PosX - StringWidth( Message )
        CenterX = False
    EndIf
    If CenterY = 2
        PosY = PosY - StringHeight( Message )
        CenterY = False
    EndIf
    Color InR,InG,InB
    Text PosX,PosY , Message , CenterX,CenterY
End Function


; A filled and shaded rectangle with upper-left light and lower-right darkness
Function DrawBevel( x,y , w,h , r,g,b , bright#=1.0,medium#=0.7,dark#=0.4 )
    Local br,bg,bb
    Local mr,mg,mb
    Local dr,dg,db
    br=r*bright:bg=g*bright:bb=b*bright
    mr=r*medium:mg=g*medium:mb=b*medium
    dr=r*dark  :dg=g*dark  :db=b*dark
    Color mr,mg,mb
    Rect x+1,y+1 , w-2,h-2 , True
    Plot x,y+h-1
    Plot x+h-1,y
    Color br,bg,bb
    Rect x,y , w-1,1 , True
    Rect x,y , 1,h-1 , True
    Color dr,dg,db
    Rect x+1,y+h-1 , w-1,1 , True
    Rect x+w-1,y+1 , 1,h-1 , True
End Function


; Draw a normal Rect, only with extra outline around/inside it
Function RectOutline( x,y , w,h , solid=False , fr=-1,fg=-1,fb=-1 , br=-1,bg=-1,bb=-1 )
; x,y: start location
; w,h: width,height
; solid: draw filled or frame
; fr,fg,fb: foreground (inside) red,green,blue
; br,bg,bb: background (outline) red,green,blue
; background (outline) is not drawn if back color not specified
; if foreground color is not specified, current color is used
    Local usefront,useback
    Local r,g,b
    
    usefront = fr >= 0 And fg >= 0 And fb >= 0
    useback  = br >= 0 And bg >= 0 And bb >= 0
    ; back color specified
    If useback
        ; front color NOT specified
        If Not usefront
            ; remember current (front) color
            r = ColorRed()
            g = ColorGreen()
            b = ColorBlue()
            ; draw back (outline)
            Color br,bg,bb
            Rect x,y , w,h , False
            ; draw inner outline if not solid
            If Not solid
                Rect x+2,y+2 , w-4,h-4 , False
            EndIf
            ; recall front color
            Color r,g,b
        ; front color specified
        Else
            ; draw back (outline)
            Color br,bg,bb
            Rect x,y , w,h , False
            ; draw inner outline if not solid
            If Not solid
                Rect x+2,y+2 , w-4,h-4 , False
            EndIf
            ; select front color
            Color fr,fg,fb
        EndIf
    ; back color NOT specified -> don't draw
    Else
        ; front color specified
        If usefront
            ; select front color
            Color fr,fg,fb
        EndIf
    EndIf
    ; draw front
    Rect x+1,y+1 , w-2,h-2 , solid
    
End Function


; Same as RectOutline, only with Oval
Function OvalOutline( x,y , w,h , solid=False , fr=-1,fg=-1,fb=-1 , br=-1,bg=-1,bb=-1 )
    Local usefront,useback
    Local r,g,b
    
    usefront = fr >= 0 And fg >= 0 And fb >= 0
    useback  = br >= 0 And bg >= 0 And bb >= 0
    If useback
        If Not usefront
            r = ColorRed()
            g = ColorGreen()
            b = ColorBlue()
            Color br,bg,bb
            Oval x,y , w,h , False
            If Not solid
                Oval x+2,y+2 , w-4,h-4 , False
            EndIf
            Color r,g,b
        Else
            Color br,bg,bb
            Oval x,y , w,h , False
            If Not solid
                Oval x+2,y+2 , w-4,h-4 , False
            EndIf
            Color fr,fg,fb
        EndIf
    Else
        If usefront
            Color fr,fg,fb
        EndIf
    EndIf
    Oval x+1,y+1 , w-2,h-2 , solid
    
End Function


;===============================================================================
;
;   Calculation routines
;
;===============================================================================


;-------------------------------------------------------------------------------
; * Word wrapped text
;-------------------------------------------------------------------------------
; return source$ cut up into pieces that fit within maxwidth
; - each piece separated by chr(0)
; - supports both fixed and variable width fonts
;-------------------------------------------------------------------------------
Function WordWrappedText$( source$ , maxwidth=-1 , mode=1 , separator$=" " )
;-------------------------------------------------------------------------------
    ; mode 0 = by character, e.g. can break in the middle of a word (separator not used)
    ; mode 1 = by word, e.g. keep all segments together as much as 'possible' (uses separator)
    If maxwidth=-1 Then maxwidth=GraphicsWidth()
    Local result$
    Local sentence$
    Local add$
    Select mode
        Case 0
            Local pos
            Local length=Len(source)
            For pos=1 To length
                add$=Mid(source,pos,1)
                If StringWidth(sentence+add)<=maxwidth
                    sentence=sentence+add
                Else
                    ;DebugLog "sentence: "+sentence
                    result=result+sentence+Chr(0)
                    sentence=add
                End If
            Next
            If sentence<>""
                result=result+sentence
                sentence=""
            End If
        Case 1
            ; trim spaces (note: also removes 'weird' chars!)
            source = Trim(source)+" "  ; cheat
            ; simplify spaces (expensive (cheap) algorithm)
            While Instr(source,"  ")
                source = Replace(source,"  "," ")
            Wend
            ; create words
            Local word$[1024]
            Local num=1
            Local thispos
            Local lastpos=1
            Repeat
                thispos=Instr(source," ",lastpos)
                If Not thispos
                    Exit
                Else
                    word[num]=Mid(source,lastpos,thispos-lastpos)
                    num=num+1
                    If num=1024 Then Exit
                    lastpos=thispos+1
                End If
            Forever
            ; walk through words
            Local count=num-1
            Local addmode=0  ; 0=word,1=char
            For num=1 To count
                ;DebugLog num+". |"+word[num]+"|"
                add=word[num]
                ;DebugLog "word: "+num+": "+add
                ; check if the word to be added fits by itself
                ; if it does not fit, keep adding pieces until done
                If StringWidth(add)>maxwidth
                    ;DebugLog "word too large"
                    length=Len(add)
                    Repeat
                        length=length-1
                    Until StringWidth(Left(add,length))<=maxwidth
                    ; stay on current word, and cut off portion
                    add=Left(add,length)
                    word[num-1]=add
                    word[num]=Mid(word[num],length+1)
                    ;DebugLog "split, add: "+add+" (to be added: "+word[num]+")"
                    num=num-1
                End If
                ; does fit
                If StringWidth(sentence+add)<=maxwidth
                    ; add to sentence
                    sentence=Trim(sentence+" "+add)
                    ;DebugLog "fits, new word added: "+sentence
                    ; next word //
                    ;num=num+1
                ; does not fit
                Else
                    ; next sentence
                    result=result+sentence+Chr(0)
                    ;DebugLog "doesn't fit, sentence complete: "+sentence
                    sentence=""
                    ; stay at same word
                    num=num-1
                    ; next sentence
                    ;num=num+1
                End If
                ; keep adding a word, until it no longer fits
                ; if it no longer fits, break to next line
            Next
            If sentence<>""
                ;DebugLog "trailing piece: "+sentence
                result=result+sentence
            End If
    End Select
    If Left(result,1)=Chr(0) Then result=Mid(result,1)
    If Right(result,1)=Chr(0) Then result=Left(result,Len(result)-1)
    Return result
;-------------------------------------------------------------------------------
End Function
;-------------------------------------------------------------------------------


; returns left-most portion of text that fits within a certain width
; - can supply desired additional string (which exceeds the width though)
;   which is only added when the width is exceeded.
Function LimitTextWidth$( source$ , width=-1 , add$=" ..." )
    If width=-1 Then width=GraphicsWidth()
    If StringWidth( source ) >= width
        Local length = Len( source )
        While StringWidth( source ) >= width
            ; keep cutting off chars 'til it fits
            length=length-1 : If length=0 Then Exit
            source = Left( source , length )
        Wend
        ;Local extra$ = " ..."
        Local extra$ = add
    Else
        extra = ""
    End If
    Return source + extra
End Function


;-------------------------------------------------------------------------------
;   RGB color objects
;-------------------------------------------------------------------------------

Type RGB
    Field r,g,b
End Type
Function NewRGB.RGB( r=0,g=0,b=0 )
    Local p.RGB
    p = New RGB
    p\r = r
    p\g = g
    p\b = b
    Return p
End Function
Function SetRGB( p.RGB , r,g,b )
    p\r = r
    p\g = g
    p\b = b
End Function
Function CopyRGB( source.RGB , target.RGB )
    target\r = source\r
    target\g = source\g
    target\b = source\b
End Function
Function DeltaRGB( start.RGB,finish.RGB , delta#=0.5 )
    Local r,g,b
    r = start\r + (finish\r - start\r) * delta
    g = start\g + (finish\g - start\g) * delta
    b = start\b + (finish\b - start\b) * delta
    Return RGB( r,g,b )
End Function
Function GetRGB( p.RGB )
    Return RGB( p\r,p\g,p\b )
End Function


;-------------------------------------------------------------------------------
;   RGB binary color conversion
;-------------------------------------------------------------------------------

; Combine Red, Green and Blue color components into a 3-byte RGB integer
Function RGB( Red,Green,Blue )
	Return Red Shl 16 Or Green Shl 8 Or Blue
End Function


; Extract individual color component from combined 3-byte RGB
Function Red( RGB )
	Return RGB Shr 16 And 255
End Function

Function Green( RGB )
	Return RGB Shr 8 And 255
End Function

Function Blue( RGB )
	Return RGB And 255
End Function


;-------------------------------------------------------------------------------
;   RGB / HSL color conversion
;-------------------------------------------------------------------------------
; Usage:
;   ; convert RGB to HSL
;   RGB_HSL 0,0,255
;
;   Print "Hue: "+hsl_h+" (flavor)"
;   Print "Saturation: "+hsl_s+" (intensity)"
;   Print "Light: "+hsl_l+" (exposure)"
;
;   ; convert HSL to RGB
;   HSL_RGB hsl_h,hsl_s,hsl_l
;
;   Print
;   Print "Red: "+rgb_r
;   Print "Green: "+rgb_g
;   Print "Blue: "+rgb_b
;
;   WaitKey
;   End
;-------------------------------------------------------------------------------
; Summary:
;   Function RGB_HSL( red,green,blue )
;   Modifies HSL_H#, HSL_S#, HSL_L#
;
;   Function HSL_RGB( hue,satura,light )
;   Modifies RGB_R, RGB_G, RGB_B
;-------------------------------------------------------------------------------
; HSL color examples:
;   Hue (flavor):
;       0.0 / 3  -  red     -  255 ,   0 ,   0
;       0.5 / 3  -  yellow  -  255 , 255 ,   0
;       1.0 / 3  -  green   -    0 , 255 ,   0
;       1.5 / 3  -  cyan    -    0 , 255 , 255
;       2.0 / 3  -  blue    -    0 ,   0 , 255
;       2.5 / 3  -  purple  -  255 ,   0 , 255
;       3.0 / 3  -  red     -  255 ,   0 ,   0
;   Saturation (intensity):
;       0.0  -    0%  -  no flavor (grey)
;       0.5  -   50%  -  half flavor (50% grey)
;       1.0  -  100%  -  full flavor (no grey)
;   Luminance / Light (exposure):
;       0.0   -  dark (black)
;       0.25  -  half brightness (50%)
;       0.5   -  full brightness (100%)
;       0.75  -  50% over-exposed (grey)
;       1.0   -  100% over-exposed (white)
;-------------------------------------------------------------------------------

Global HSL_H#   ; hue (color flavor: 0..1)
Global HSL_S#   ; saturation (flavor intensity: 0..1)
Global HSL_L#   ; light (exposure level: 0..1)
Function RGB_HSL( Red,Green,Blue )
;   Param Red: 0..255
;   Param Green: 0..255
;   Param Blue: 0..255
    Local r#,g#,b#
    Local min#,max#,dif#
    Local r2#,g2#,b2#

    ; scale to 0..1
    r = Red / 255.
    g = Green / 255.
    b = Blue / 255.

    ; find minimum
    min = r
    If g < min
        min = g
    EndIf
    If b < min
        min = b
    EndIf

    ; find maximum
    max = r
    If g > max
        max = g
    EndIf
    If b > max
        max = b
    EndIf

    ; find difference between minimum and maximum
    dif = max - min

    ; light level
    HSL_L = ( max + min ) / 2.

    ; no flavor
    If dif = 0.
        HSL_H = 0.
        HSL_S = 0.
    ; any flavor
    Else
        ; determine saturation
        ; - light level below 50%
        If HSL_L < 0.5
            HSL_S = dif / ( max + min )
        ; - light level above 50%
        Else
            HSL_S = dif / ( 2. - max - min )
        EndIf

        ; determine hue
        ; - red is maximum
        If r = max
            HSL_H =         (g - b) / dif / 6.
        ; - green is maximum
        ElseIf g = max
            HSL_H = 1./3. + (b - r) / dif / 6.
        ; - blue is maximum
        ElseIf b = max
            HSL_H = 2./3. + (r - g) / dif / 6.
        EndIf

    EndIf

End Function


Global RGB_R
Global RGB_G
Global RGB_B
Function HSL_RGB( Hue#,Satura#,Light# )
;   Param Hue: color flavor (0..1)
;   Param Satura: flavor intensity (0..1)
;   Param Light: exposure level (0..1)
    Local low#,high#

    ; no flavor
    If Satura = 0.
        RGB_R = Light * 255.
        RGB_G = Light * 255.
        RGB_B = Light * 255.
    ; any flavor
    Else
        ; high level
        ; - light level below 50%
        If Light < 0.5
            high = Light * (1. + Satura)
        ; - light level above 50%
        Else
            high = (Light + Satura) - (Satura * Light)
        EndIf

        ; low level
        low = 2. * Light - high

        ; color level
        RGB_R = HSL_RGB_Hue( low,high , Hue + 1./3. ) * 255.
        RGB_G = HSL_RGB_Hue( low,high , Hue         ) * 255.
        RGB_B = HSL_RGB_Hue( low,high , Hue - 1./3. ) * 255.
    EndIf

End Function

; sub-routine
Function HSL_RGB_Hue#( low# , high# , hue# )

    If hue < 0.
        hue = hue + 1.
    ElseIf hue > 1.
        hue = hue - 1.
    EndIf

    If 6. * hue < 1.
        Return low + (high-low) * hue * 6.
    ElseIf 2. * hue < 1.
        Return high
    ElseIf 3. * hue < 2.
        Return low + (high-low) * (2./3. - hue) * 6.
    Else
        Return low
    EndIf

End Function


;~IDEal Editor Parameters:
;~C#Blitz3D1105