; self-initializing module (light),
; all other components are call-only.
;-------------------------------------------------------------------------------
; this module is intended to serve
; the same purpose as 'render.lib \ colortext()',
; only a lot faster,
; using separate calls.
; for usage see demo function.
;-------------------------------------------------------------------------------
; - to align a fully colorized string.
;   use 'render.lib \ colortext()'.
; - to print text using 1 command in 1 color
;   use 'render.lib \ textoutline()'.
;-------------------------------------------------------------------------------

Global Text2D_PosX
Global Text2D_PosY
Global Text2D_fr        ; front red (inside)
Global Text2D_fg        ;       green
Global Text2D_fb        ;       blue
Global Text2D_br        ; back  red (outline)
Global Text2D_bg        ;       green
Global Text2D_bb        ;       blue
Global Text2D_Sep = 1   ; separation (between inside and outline)
Global Text2D_drawback = False  ; draw outline at all (color is set)
Global Text2D_setfront = False  ; inside color is set (manual instead of color)


; set cursor for consecutive drawing (Write)
Function Text2D_Locate( x,y )
    
    Text2D_PosX = x
    Text2D_PosY = y
    
End Function


; relative cursor adjustment (by pixel)
Function Text2D_Move( dx=0,dy=0 )
    
    Text2D_PosX = Text2D_PosX + dx
    Text2D_PosY = Text2D_PosY + dy
    
End Function


; relative cursor adjustment (by string)
Function Text2D_MoveX( dx$="" , dir#=1 )
    
    Text2D_PosX = Text2D_PosX + StringWidth( dx ) * dir
    
End Function


; relative cursor adjustment (by fontheight)
Function Text2D_MoveY( dy#=1 )
    
    Text2D_PosY = Text2D_PosY + dy * FontHeight()
    
End Function


; set foreground (inside) color
Function Text2D_Front( fr,fg,fb )
    
    Text2D_setfront = False
    If fr <> -1
        If fg <> -1
            If fb <> -1
                Text2D_setfront = True
                Text2D_fr = fr
                Text2D_fg = fg
                Text2D_fb = fb
            EndIf
        EndIf
    EndIf
    
End Function


; set background (outline) color
Function Text2D_Back( br=-1,bg=-1,bb=-1 )
    
    Text2D_drawback = False
    If br <> -1
        If bg <> -1
            If bb <> -1
                Text2D_drawback = True
                Text2D_br = br
                Text2D_bg = bg
                Text2D_bb = bb
            EndIf
        EndIf
    EndIf
    
End Function


; set separation/distance between inside and outline (in pixels)
Function Text2D_Sep( separation )
    
    Text2D_Sep = separation
    
End Function


; write something, then place/keep cursor right behind it
Function Text2D_Write( msg$ , cx=0,cy=0 , cursor_stay=False )
;   Param cx,cy      ; center x,y (alignment): 0 = left/top, 1 = center, 2 = right/bottom
;X   Param fr,fg,fb   ; front color (inside)
;X   Param br,bg,bb   ; back color (outline)
;X   Param separation ; distance between outline and inside in pixels
    ;Local drawback
    ;Local setfront
    Local drawx,drawy
    Local sizex,sizey
    Local bx,by
    Local fr,fg,fb
    
    ; horizontal alignment
    Select cx
        Case 0  ; left align
            drawx = Text2D_PosX
            If Not cursor_stay
                ; place cursor behind text
                sizex = StringWidth( msg )
                Text2D_PosX = Text2D_PosX + sizex
            EndIf
        Case 1  ; center
            sizex = StringWidth( msg )
            drawx = Text2D_PosX - sizex/2
            If Not cursor_stay
                ; place cursor behind text
                Text2D_PosX = Text2D_PosX + sizex/2
            EndIf
        Case 2  ; right align
            sizex = StringWidth( msg )
            drawx = Text2D_PosX - sizex
            ; no change in cursor position
    End Select
    ; vertical alignment
    Select cy
        Case 0  ; top align
            sizey = StringWidth( msg )
            drawy = Text2D_PosY
        Case 1  ; center
            sizey = StringWidth( msg )
            drawy = Text2D_PosY - sizey/2
        Case 2  ; bottom align
            sizey = StringWidth( msg )
            drawy = Text2D_PosY - sizey
    End Select
    
    ; draw outline if background color set
    ;If Text2D_br <> -1
    ;    If Text2D_bg <> -1
    ;        If Text2D_bb <> -1
    ;            drawback = True
    ;        EndIf
    ;    EndIf
    ;EndIf
    ; change inside color if front color set
    ;If Text2D_fr <> -1
    ;    If Text2D_fg <> -1
    ;        If Text2D_fb <> -1
    ;            setfront = True
    ;        EndIf
    ;    EndIf
    ;EndIf
    
    ; draw outline too
    If Text2D_drawback
        ; front color not set, keep current
        If Not Text2D_setfront
            fr = ColorRed()
            fg = ColorGreen()
            fb = ColorBlue()
        EndIf
        Color Text2D_br,Text2D_bg,Text2D_bb
        For bx = -1 To 1
            For by = -1 To 1
                If bx <> 0
                    If by <> 0
                        Text drawx+bx*Text2D_Sep,drawy+by*Text2D_Sep , msg
                    EndIf
                EndIf
            Next
        Next
    EndIf
    If Text2D_setfront
        Color Text2D_fr,Text2D_fg,Text2D_fb
    End If
    Text drawx,drawy , msg
    
End Function


; write something, then place/keep cursor right behind it
Function Text2D_Write_( msg$ , cx=0,cy=0 , fr=-1,fg=-1,fb=-1 , br=-1,bg=-1,bb=-1 , separation=1 )
;   Param cx,cy      ; center x,y (alignment): 0 = left/top, 1 = center, 2 = right/bottom
;   Param fr,fg,fb   ; front color (inside)
;   Param br,bg,bb   ; back color (outline)
;   Param separation ; distance between outline and inside in pixels
    Local drawback
    Local setfront
    Local drawx,drawy
    Local sizex,sizey
    Local bx,by
    
    ; horizontal alignment
    Select cx
        Case 0  ; left align
            sizex = StringWidth( msg )
            drawx = Text2D_PosX
            ; place cursor behind text
            Text2D_PosX = Text2D_PosX + sizex
        Case 1  ; center
            sizex = StringWidth( msg )
            drawx = Text2D_PosX - sizex/2
            ; place cursor behind text
            Text2D_PosX = Text2D_PosX + sizex/2
        Case 2  ; right align
            sizex = StringWidth( msg )
            drawx = Text2D_PosX - sizex
            ; no change in cursor position
    End Select
    ; vertical alignment
    Select cy
        Case 0  ; top align
            sizey = StringWidth( msg )
            drawy = Text2D_PosY
        Case 1  ; center
            sizey = StringWidth( msg )
            drawy = Text2D_PosY - sizey/2
        Case 2  ; bottom align
            sizey = StringWidth( msg )
            drawy = Text2D_PosY - sizey
    End Select
    
    ; draw outline if background color set
    If br <> -1
        If bg <> -1
            If bb <> -1
                drawback = True
            EndIf
        EndIf
    EndIf
    ; change inside color if front color set
    If fr <> -1
        If fg <> -1
            If fb <> -1
                setfront = True
            EndIf
        EndIf
    EndIf
    
    ; draw outline too
    If drawback
        ; front color not set, keep current
        If Not setfront
            fr = ColorRed()
            fg = ColorGreen()
            fb = ColorBlue()
        EndIf
        Color br,bg,bb
        For bx = -1 To 1
            For by = -1 To 1
                If bx <> 0
                    If by <> 0
                        Text drawx+bx*separation,drawy+by*separation , msg
                    EndIf
                EndIf
            Next
        Next
    EndIf
    ; don't draw outline, just the inside (normal text)
;    Else
        If setfront
            Color fr,fg,fb
        End If
        Text drawx,drawy , msg
 ;   EndIf
    
End Function


; demonstration of system
Function Text2D_Demo()
    Local sw = 320
    Local sh = 240
    
    Graphics sw,sh , 0,3
    SetFont LoadFont( "System" , 50 )
    SetBuffer BackBuffer()
    Cls
    Text2D_Sep 1
    ; top: front only
    Text2D_Locate sw/2,sh/2
    Text2D_MoveX "NICE" , -.5
    Text2D_MoveY -.5-1
    Color 255,255,255
    Text2D_Write "N"
    Color 255,255,0
    Text2D_Write "I"
    Color 255,128,0
    Text2D_Write "C"
    Color 255,0,0
    Text2D_Write "E"
    ; bottom: front and back
    Text2D_Front 0,0,0
    Text2D_Locate sw/2,sh/2
    Text2D_MoveX "NICE" , -.5
    Text2D_MoveY -.5+1
    Text2D_Back 0,0,255
    Text2D_Write "N"
    Text2D_Back 0,128,255
    Text2D_Write "I"
    Text2D_Back 0,255,255
    Text2D_Write "C"
    Text2D_Back 255,255,255
    Text2D_Write "E"
    Flip
    WaitKey
    End
    
End Function


;Text2D_Demo

;~IDEal Editor Parameters:
;~C#Blitz3D