
;-------------------------------------------------------------------------------
; * WinMouse - Locked, protected and relative mouse positioning in a window
; Version 04 Mar 2007
; Created by Teasy <TheChange@yahoo.com>
;-------------------------------------------------------------------------------
; UserLibs:
;   user32.decls
;       .lib "user32.dll"
;       api_GetCursorPos( pt* ) : "GetCursorPos"
;-------------------------------------------------------------------------------

Include "\Dev\Blitz\Lib\WindowHandling.BB"  ; more api calls


Type Point2D
    Field X,Y
End Type

Type WinMouse_Object
    Field Pos.Point2D
    Field LastCall         ; time last called update (for calibration)
    Field CenterX,CenterY  ; center of screen (calibrated)
    Field MouseX,MouseY    ; current in-window mouse position
    Field DeskW,DeskH      ; desktop dimensions
End Type


Global WinMouse.WinMouse_Object


Function WinMouse_Init()
    WinMouse = New WinMouse_Object
    WinMouse\Pos = New Point2D
    ; put 1 minute in the past
    WinMouse\LastCall = MilliSecs() - 60000
    ; calibrate center
    WinMouse_Update
    ; set blitz cursor to center
    WinMouse\MouseX = GraphicsWidth()/2
    WinMouse\MouseY = GraphicsHeight()/2
    ; obtain desktop dimensions
    WinMouse\DeskW = DesktopWidth()
    WinMouse\DeskH = DesktopHeight()
End Function


Function WinMouse_Exit()
    Delete WinMouse\Pos
    Delete WinMouse
End Function


Function WinMouse_Update( AbsOnly = True , IdleTime = 500 )
;   Param AbsOnly   ; True: Update only Windows mouse position
;                   ; False: Locked mouse motion
;   Param IdleTime  ; Amount of ms before recalibrating
;                   ; when not calling this function.
    Local MilliSecs       ; number of ms since boot
    Local GraphicsWidth   ; blitz window dimensions
    Local GraphicsHeight
    Local SpeedX#,SpeedY#  ; mouse speed factor (for different window sizes)
; Note: calibration is required when the window size changes.
; You can manually calibrate using WinMouse_Calibrate,
; or using an IdleTime of 0 or below.

    If Not AbsOnly
        ; precall
        MilliSecs = MilliSecs()
        GraphicsWidth = GraphicsWidth()
        GraphicsHeight = GraphicsHeight()

        ; last call was longer than 0.5 second ago
        If MilliSecs - WinMouse\LastCall >= IdleTime
            ; recalibrate mouse center
            ; - center on screen
            MoveMouse GraphicsWidth/2,GraphicsHeight/2
            ; - obtain absolute mouse position
            api_GetCursorPos WinMouse\Pos
            ; - store it
            WinMouse\CenterX = WinMouse\Pos\X
            WinMouse\CenterY = WinMouse\Pos\Y
        EndIf
        ; update time of call
        WinMouse\LastCall = MilliSecs
    EndIf 

    ; get only the position
    If AbsOnly
        ; retrieve only the absolute mouse position
        api_GetCursorPos WinMouse\Pos
    ; move a virtual mouse while locking the real one (protected)
    Else 
        ; detect mouse motion
        ; - retrieve absolute mouse position (again)
        api_GetCursorPos WinMouse\Pos
        ; - position the mouse center of window
        MoveMouse GraphicsWidth/2,GraphicsHeight/2
        ; - check distance travelled (with calibrated center)
        ;   and add it to the current position.
;        WinMouse\MouseX = WinMouse\MouseX + ( WinMouse\Pos\X - WinMouse\CenterX )
;        WinMouse\MouseY = WinMouse\MouseY + ( WinMouse\Pos\Y - WinMouse\CenterY )
        ;   Now using desktop / active window dimensions to scale the motion
        ;   to keep the mouse movement speed (sensivity) 
        ;   the same for any window size
        SpeedX = Float WinMouse\DeskW / Float ActiveWindowWidth() 
        SpeedY = Float WinMouse\DeskH / Float ActiveWindowHeight()
        WinMouse\MouseX = WinMouse\MouseX + Float ( WinMouse\Pos\X - WinMouse\CenterX ) * SpeedX;/2
        WinMouse\MouseY = WinMouse\MouseY + Float ( WinMouse\Pos\Y - WinMouse\CenterY ) * SpeedY;/2
        ; - limit position to screen boundaries
        If WinMouse\MouseX <               0 Then WinMouse\MouseX = 0
        If WinMouse\MouseX >=  GraphicsWidth Then WinMouse\MouseX = GraphicsWidth-1
        If WinMouse\MouseY <               0 Then WinMouse\MouseY = 0
        If WinMouse\MouseY >= GraphicsHeight Then WinMouse\MouseY = GraphicsHeight-1
    EndIf 
End Function


; force calibration (in case the window size is changed within the idle time)
Function WinMouse_Calibrate()
    ; recalibrate mouse center
    ; - center on screen
    MoveMouse GraphicsWidth()/2,GraphicsHeight()/2
    ; - obtain absolute mouse position
    api_GetCursorPos WinMouse\Pos
    ; - store it
    WinMouse\CenterX = WinMouse\Pos\X
    WinMouse\CenterY = WinMouse\Pos\Y
End Function

;-------------------------------------------------------------------------------
; Demonstration

Function WinMouse_Demo()
    WinMouse_Init
    
    Local sx = 320
    Local sy = 240
    
    Graphics sx,sy , 0,3
    SetBuffer BackBuffer()
    
    SetFont LoadFont( "Arial" , 15 , True )
    
    Repeat
        WinMouse_Update KeyDown( 57 )
        If KeyHit( 28 )
            FlushKeys 
            WaitKey 
            FlushKeys 
        EndIf 
        Cls
        Color 255,192,128
        Rect WinMouse\MouseX-10,WinMouse\MouseY-10 , 20,20 , False 
        Rect WinMouse\MouseX- 3,WinMouse\MouseY- 3 ,  6, 6 , True 
        Color 128,192,255
        Text 16,16 , "(Blitz) Mouse: ("+MouseX()  +","+  MouseY()+")"
        Text 16,32 , "(Windows) Mouse: ("+WinMouse\Pos\X+","+WinMouse\Pos\Y+")"
        Text 16,48 , "(WinMouse) Mouse: ("+WinMouse\MouseX+","+WinMouse\MouseY+")"
        Text 16,64 , "(Center) Mouse: ("+WinMouse\CenterX+","+WinMouse\CenterY+")"
        Flip
    Until KeyHit( 1 )
    
    WinMouse_Exit
    
    End
End Function

;winmouse_demo
