
;-------------------------------------------------------------------------------
; * Window Handling
;-------------------------------------------------------------------------------
; Created by Teasy <TheChange@yahoo.com>
; With divine help from EPS, Yan and Rufus Serano
;-------------------------------------------------------------------------------
; Use Windows' native system functions to change a window's properties,
; such as its dimensions, state and visual make-up.
;-------------------------------------------------------------------------------
; 06 Aug 2012
; - Added SetWindowIcon to change the runtime window icon
; 08 Dec 2011
; - Added posx/y for desktop and graphicswindow (4 functions)
; - Added resize and move for graphicswindow (2 functions) (includes centering)
; 08 Nov 2011
; - Added Window_DisableCloseButton
; 18 Aug 2011
; - Added Maximize, Minimize and Restore Graphics Window functions
; 22 Dec 2010
; - Changed layout of code
; - Added GraphicsWindowWidth and GraphicsWindowHeight
;   - Uses "AppHWND" for safer window access
; - Added GraphicsWindow() which wraps Graphics() with a new mode (4)
;   - Mode 4 turns the window into a fake fullscreen (scaled)
;-------------------------------------------------------------------------------

;,
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;,,, , ,
;;;,d%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&b.
;,d&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%b.
;&%&%&%&'
;%&%&%&%    Window state
;&%&%&%&,
;'V&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%P`
;;;'V%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&P`
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;:::  
;'

Const SW_HIDE            =  0
Const SW_SHOWNORMAL      =  1
Const SW_NORMAL          =  1
Const SW_SHOWMINIMIZED   =  2
Const SW_SHOWMAXIMIZED   =  3
Const SW_MAXIMIZE        =  3
Const SW_SHOWNOACTIVATE  =  4
Const SW_SHOW            =  5
Const SW_MINIMIZE        =  6
Const SW_SHOWMINNOACTIVE =  7
Const SW_SHOWNA          =  8
Const SW_RESTORE         =  9
Const SW_SHOWDEFAULT     = 10
Const SW_MAX             = 11

;-------------------------------------------------------------------------------

; Change the active window, e.g. Blitz
Function ShowWindow( message = SW_NORMAL )
	Local window_handle = api_GetActiveWindow()
	api_ShowWindow( window_handle , message )
End Function

;Function ShowWindow_Demo()
;    Graphics 512,384 , 0,3
;    Print "untouched" : WaitKey : Cls 
;    ShowWindow SW_MAXIMIZE
;    Print "maximized" : WaitKey : Cls 
;    ShowWindow SW_RESTORE
;    Print "restored" : WaitKey : Cls 
;    End 
;End Function

;-------------------------------------------------------------------------------
; * Active window functions
;-------------------------------------------------------------------------------
; User32.decls:
;   .lib "USER32.DLL"
;   API_ShowWindow( hWnd% , nCmdShow% ) : "ShowWindow"
;   API_GetActiveWindow%() : "GetActiveWindow"
;-------------------------------------------------------------------------------

Function MaximizeActiveWindow()
    api_ShowWindow api_GetActiveWindow() , 3
End Function
Function MinimizeActiveWindow()
    api_ShowWindow api_GetActiveWindow() , 6
End Function
Function RestoreActiveWindow()
    api_ShowWindow api_GetActiveWindow() , 9
End Function

Function MaximizeGraphicsWindow()
    api_ShowWindow SystemProperty( "AppHWND" ) , 3
End Function
Function MinimizeGraphicsWindow()
    api_ShowWindow SystemProperty( "AppHWND" ) , 6
End Function
Function RestoreGraphicsWindow()
    api_ShowWindow SystemProperty( "AppHWND" ) , 9
End Function

;,
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;,,, , ,
;;;,d%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&b.
;,d&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%b.
;&%&%&%&'
;%&%&%&%    Window modification
;&%&%&%&,
;'V&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%P`
;;;'V%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&P`
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;:::  
;'

;-------------------------------------------------------------------------------
; * userlibs
;-------------------------------------------------------------------------------
; .lib "USER32.DLL"
; api_GetSystemMenu%( hWnd% , bRevert% ) : "GetSystemMenu"
; api_DeleteMenu%( hMenu% , uPos% , uFlags% ) : "DeleteMenu"
;-------------------------------------------------------------------------------

Function Window_DisableCloseButton( hWnd=0 )
    If hWnd = 0
        hWnd = SystemProperty( "AppHWND" )
    End If
    Local hMenu = api_GetSystemMenu( hWnd , False )
    If hMenu
        Local SC_CLOSE = 61536
        api_DeleteMenu hMenu , SC_CLOSE , 0
    End If
End Function

;-------------------------------------------------------------------------------
; * userlibs
;-------------------------------------------------------------------------------
; .lib "shell32.dll"
; api_ExtractIcon%( hWnd% , File$ , Index% ) : "ExtractIconA"
;-------------------------------------------------------------------------------
; .lib "user32.dll"
; api_SetClassLong%( hWnd% , nIndex% , Value% ) : "SetClassLongA"
;-------------------------------------------------------------------------------

; example: Window_SetIcon "c:\windows\explorer.exe"
Function Window_SetIcon( IconFile$,IconIndex=0 , hWnd=0 )
    If hWnd = 0
        hWnd = SystemProperty( "AppHWND" )
    End If
    ; extract (replicate) icon for use
	Local hIcon = api_ExtractIcon( hWnd , IconFile$ , IconIndex )
    ; assign icon to window
	api_SetClassLong hWnd , -14 , hIcon
End Function

;,
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;,,, , ,
;;;,d%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&b.
;,d&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%b.
;&%&%&%&'
;%&%&%&%    Window dimensions
;&%&%&%&,
;'V&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%P`
;;;'V%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&P`
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;:::  
;'

Type struct_POINT
    Field X,Y
End Type

Type struct_RECT
	Field Left
	Field Top
	Field Right
	Field Bottom
End Type

Global struct_POINT.struct_POINT = New struct_POINT
Global struct_RECT.struct_RECT = New struct_RECT

;-------------------------------------------------------------------------------
; userlibs:
;   user32.decls
;     .lib "user32.dll"
;     api_GetDesktopWindow%() : "GetDesktopWindow"
;     api_GetWindowRect%( hwnd% , lpRect* ) : "GetWindowRect"
;-------------------------------------------------------------------------------

Function ActiveWindowWidth()
   api_GetWindowRect api_GetActiveWindow() , struct_RECT
   Return struct_RECT\Right - struct_RECT\Left
End Function 
Function ActiveWindowHeight()
   api_GetWindowRect api_GetActiveWindow() , struct_RECT
   Return struct_RECT\Bottom - struct_RECT\Top
End Function 

Function DesktopPosX()
    api_GetWindowRect api_GetDesktopWindow() , struct_RECT
    Return struct_RECT\Left
End Function
Function DesktopPosY()
    api_GetWindowRect api_GetDesktopWindow() , struct_RECT
    Return struct_RECT\Top
End Function
Function DesktopWidth()
   api_GetWindowRect api_GetDesktopWindow() , struct_RECT
   Return struct_RECT\Right - struct_RECT\Left
End Function 
Function DesktopHeight()
   api_GetWindowRect api_GetDesktopWindow() , struct_RECT
   Return struct_RECT\Bottom - struct_RECT\Top
End Function 

Function GraphicsWindowPosX()
    api_GetWindowRect SystemProperty("AppHWND") , struct_RECT
    Return struct_RECT\Left
End Function
Function GraphicsWindowPosY()
    api_GetWindowRect SystemProperty("AppHWND") , struct_RECT
    Return struct_RECT\Top
End Function
Function GraphicsWindowWidth()
    api_GetWindowRect SystemProperty( "AppHWND" ) , struct_RECT
    Return struct_RECT\Right - struct_RECT\Left
End Function
Function GraphicsWindowHeight()
    api_GetWindowRect SystemProperty( "AppHWND" ) , struct_RECT
    Return struct_RECT\Bottom - struct_RECT\Top
End Function

; the position is always equal to 0,0
Function GraphicsClientPosX()
    api_GetClientRect SystemProperty("AppHWND") , struct_RECT
    Return struct_RECT\Left
End Function
Function GraphicsClientPosY()
    api_GetClientRect SystemProperty("AppHWND") , struct_RECT
    Return struct_RECT\Top
End Function
; if the window is scaled, these may become different from the blitz 'graphicswindow' width and height
Function GraphicsClientWidth()
    api_GetClientRect SystemProperty( "AppHWND" ) , struct_RECT
    Return struct_RECT\Right - struct_RECT\Left
End Function
Function GraphicsClientHeight()
    api_GetClientRect SystemProperty( "AppHWND" ) , struct_RECT
    Return struct_RECT\Bottom - struct_RECT\Top
End Function

; these return the absolute desktop coordinates of the drawing 'canvas' (in blitz3d)
; - e.g. the (0,0) offset by the window position and its window frame, caption/title/menu bar etc
Function GraphicsCanvasPosX()
    struct_POINT\X = 0
    struct_POINT\Y = 0
    api_ClientToScreen SystemProperty( "AppHWND" ) , struct_POINT
    Return struct_POINT\X
End Function
Function GraphicsCanvasPosY()
    struct_POINT\X = 0
    struct_POINT\Y = 0
    api_ClientToScreen SystemProperty( "AppHWND" ) , struct_POINT
    Return struct_POINT\Y
End Function

;-------------------------------------------------------------------------------
; * Older code (low level hacks) by EPS
;-------------------------------------------------------------------------------
;Function DesktopWidth()  ; by EPS
;;======================================== 
;; Ermittle Windows Desktopbreite 
;;======================================== 
;   Local struct_rect% = CreateBank(16) 
;
;   api_GetWindowRect(api_GetDesktopWindow(),struct_rect) 
;   Local res = PeekInt(struct_rect,8) 
;   FreeBank struct_rect 
;
;   Return res 
;End Function 
;
;
;Function DesktopHeight()  ; by EPS
;;======================================== 
;; Ermittle Windows Desktophhe 
;;======================================== 
;   Local struct_rect% = CreateBank(16) 
;
;   api_GetWindowRect(api_GetDesktopWindow(),struct_rect) 
;   Local res = PeekInt(struct_rect,12) 
;   FreeBank struct_rect 
;
;   Return res 
;End Function 
;-------------------------------------------------------------------------------

Function WindowDimensions_Demo()
    Graphics 160,120 , 0,3
    SetBuffer BackBuffer()
    Repeat 
        Cls 
        Text 0,0  , "Canvas: "+GraphicsWidth()+"x"+GraphicsHeight()
        Text 0,10 , "Window: "+ActiveWindowWidth()+"x"+ActiveWindowHeight()
        Text 0,20 , "Desktop: "+DesktopWidth()+"x"+DesktopHeight()
        Flip 
    Until KeyHit( 1 )
    End 
End Function

;-------------------------------------------------------------------------------

Function ResizeGraphicsWindow( w,h , center=False )
    Local x,y
    If center
        ; actually this doesn't work if used when the window is placed
        ; on the non-primary monitor; (the window will be moved to the primary monitor)
        ; so perhaps seek the upperleft pos of the current monitor,
        ; e.g. of the desktop under the client window
        Local dx = DesktopPosX()
        Local dy = DesktopPosY()
        Local dw = DesktopWidth()
        Local dh = DesktopHeight()
        x = dx + dw/2 - w/2
        y = dy + dh/2 - h/2
    Else
        Local gx = GraphicsWindowPosX()
        Local gy = GraphicsWindowPosY()
        ;Local gw = GraphicsWindowWidth()
        ;Local gh = GraphicsWindowHeight()
        x = gx
        y = gy
    End If
    api_MoveWindow SystemProperty("AppHWND") , x,y , w,h , True
End Function
Function MoveGraphicsWindow( x,y )
    Local gw = GraphicsWindowWidth()
    Local gh = GraphicsWindowHeight()
    api_MoveWindow SystemProperty("AppHWND") , x,y , gw,gh , True
End Function

;,
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;,,, , ,
;;;,d%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&b.
;,d&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%b.
;&%&%&%&'
;%&%&%&%    Fake full-screen
;&%&%&%&,
;'V&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%P`
;;;'V%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&%&P`
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;:::  
;'

;-------------------------------------------------------------------------------
; GraphicsWindow( pixel_width , pixel_height , color_depth , window_mode )
; - pixel_width, pixel_height, color_depth: same as Graphics
; - window_mode: same as Graphics, except mode 4; which is 'fake fullscreen'.
;-------------------------------------------------------------------------------
; File: api.User32.decls
;   .lib "USER32.DLL"
;   api_GetClientRect( hwnd% , lprect* ) : "GetClientRect"
;   api_GetWindowLong% ( hwnd% , index% ) : "GetWindowLongA"
;   api_SetWindowLong% ( hwnd% , index% , info% ) : "SetWindowLongA"
;   api_SetWindowPos% ( hWnd% , Depth% , x1%,y1%,x2%,y2% , Actions% ) : "SetWindowPos"
;-------------------------------------------------------------------------------
Const GraphicsWindow_SilentMode = False
Type GraphicsWindow_Rect
    Field x1,y1
    Field x2,y2
End Type
Function GraphicsWindow( width,height , depth=32,mode=4 )
    If width <= 0 Or height <= 0
        GraphicsWindow_Error "Screen dimensions out of range: "+width+"x"+height
    End If
    Select depth
        Case 0,8,16,24,32
        Default
            GraphicsWindow_Error "Invalid color depth: "+depth
            Return False
    End Select
    Select mode
        Case 0,1,2,3  ; 0: auto, 1: full, 2: window, 3: scaled window
            Graphics width,height , depth,mode
        Case 4
            GraphicsWindow_FakeFullscreen width,height , depth
        Default
            GraphicsWindow_Error "Undefined window mode: "+mode
            Return False
    End Select
    Return True
End Function
Function GraphicsWindow_FakeFullscreen( w,h , d )
    ; setup blitz graphics
    Graphics w,h , d,3
    
    ; change window properties
    Local hwin = SystemProperty( "AppHWND" )  ; QueryObject( win , 1 )  ; blitzplus
    ; assume window has caption and frame -> toggle them off
    Local style = api_GetWindowLong( hwin , -16 ) Xor ($C00000+$40000)
    api_SetWindowLong hwin , -16 , style
    
    ; change window dimensions
    Local deskrect.GraphicsWindow_Rect = New GraphicsWindow_Rect
    api_GetClientRect api_GetDesktopWindow() , deskrect
    api_SetWindowPos hwin,0 , 0,0,deskrect\x2,deskrect\y2 , 4
    Delete deskrect
    
    ;api_ShowWindow hwin , 3  ; maximize
End Function
Function GraphicsWindow_Error( message$ )
    If GraphicsWindow_SilentMode
        DebugLog "() GraphicsWindow - "+message
    Else
        RuntimeError "() GraphicsWindow:"+Chr(13)+message
    End If
End Function

;-------------------------------------------------------------------------------

;~IDEal Editor Parameters:
;~C#Blitz3D1105